#include <windows.h>

#include "debug.h"

#include "protodef.h"
#include "commproto.h"

BOOL SendRequest(struct CommDev *lpDevice, LPCVOID lpRequest, WORD wRequestSize)
{
	ASSERT(lpDevice);
	ASSERT(lpRequest);
	ASSERT(wRequestSize > 0);

	BOOL fFailed = FALSE;

	// Send low byte of byte count
	if (!fFailed && CommDevWriteByte(lpDevice, LOBYTE(wRequestSize)) == FALSE)
		fFailed = TRUE;

	// Send high byte of byte count
	if (!fFailed && CommDevWriteByte(lpDevice, HIBYTE(wRequestSize)) == FALSE)
		fFailed = TRUE;

	// Send bytes from request buffer
	if (!fFailed)
	{
		const BYTE *lpData = lpRequest;

		WORD wCurrPos = 0;
		while (!fFailed && wCurrPos < wRequestSize)
		{
			if (CommDevWriteByte(lpDevice, lpData[wCurrPos++]) == FALSE)
			{
				fFailed = TRUE;
			}
		}
	}

	return !fFailed;
}

BOOL ReceiveResponse(struct CommDev *lpDevice, LPVOID lpResponse, LPWORD lpResponseSize)
{
	ASSERT(lpDevice);
	ASSERT(lpResponse);
	ASSERT(lpResponseSize > 0);

	BOOL fFailed = FALSE;
	WORD wBytesToRead;

	// Get low byte of byte count
	if (!fFailed)
	{
		BYTE bByte;
		if (CommDevReadByte(lpDevice, &bByte))
			wBytesToRead = bByte;
		else
			fFailed = TRUE;
	}

	// Get high byte of byte count
	if (!fFailed)
	{
		BYTE bByte;
		if (CommDevReadByte(lpDevice, &bByte))
			wBytesToRead = MAKEWORD(wBytesToRead, bByte);
		else
			fFailed = TRUE;
	}

	// Read specified number of bytes, copy as much as possible
	if (!fFailed)
	{
		BYTE *lpData = lpResponse;
		const WORD wMaxBytesToCopy = *lpResponseSize;

		WORD wCurrPos = 0;
		while (!fFailed && wCurrPos < wBytesToRead)
		{
			BYTE bByte;
			if (CommDevReadByte(lpDevice, &bByte))
			{
				if (wCurrPos < wMaxBytesToCopy)
				{
					lpData[wCurrPos++] = bByte;
				}
				else
				{
					// Silently discard bytes to keep protocol intact
				}
			}
			else
			{
				fFailed = TRUE;
			}
		}

		*lpResponseSize = wCurrPos;
	}

	return !fFailed;
}

BOOL ProtocolTransaction(struct CommDev *lpDevice, LPCVOID lpRequest, WORD wRequestSize, LPVOID lpResponse, LPWORD lpResponseSize)
{
	ASSERT(lpDevice);
	ASSERT(lpRequest);
	ASSERT(wRequestSize > 0);
	ASSERT(lpResponse);
	ASSERT(lpResponseSize && *lpResponseSize > 0);

	BOOL fFailed = FALSE;

	// Send request
	if (!fFailed && SendRequest(lpDevice, lpRequest, wRequestSize) == FALSE)
	{
		TRACE("WriteRequest failed.\n");
		fFailed = TRUE;
	}

	// Receive response
	if (!fFailed && ReceiveResponse(lpDevice, lpResponse, lpResponseSize) == FALSE)
	{
		TRACE("ReadResponse failed.\n");
		fFailed = TRUE;
	}

	// Check if response code matches request code + success flag
	if (!fFailed)
	{
		const BYTE bRequestCode = *((BYTE*) lpRequest);
		const BYTE bResponseCode = *((BYTE*) lpResponse);
		if (bResponseCode != (bRequestCode | RES_FLAG_SUCCESS))
		{
			TRACE("Toolbox responded with unexpected code: 0x%.2x\n", bResponseCode);
			fFailed = TRUE;
		}
	}

	return !fFailed;
}
