#include <stdio.h>
#include <stdlib.h>
#include <windows.h>

#include "settings.h"
#include "resource.h"
#include "guiutils.h"

#include "debug.h"

#include "convertdlg.h"

#define MAX_VALUE 50

static void UpdateControls(HWND hDlg, const struct ConvertDlgData *lpData)
{
	ShowWindow(GetDlgItem(hDlg, IDC_DATALOSS_WARNING), lpData->dwTargetDriveCapacity < lpData->dwSourceImageSize ? SW_SHOW : SW_HIDE);

	CheckDlgButton(hDlg, IDC_REWRITE_FAT12, lpData->fRewriteFat12 ? BST_CHECKED : BST_UNCHECKED);
	EnableWindow(GetDlgItem(hDlg, IDC_SOURCE_IMAGE_GROUPBOX), lpData->fRewriteFat12);
	EnableWindow(GetDlgItem(hDlg, IDC_SOURCE_NUMBER_OF_SECTORS_LABEL), lpData->fRewriteFat12);
	EnableWindow(GetDlgItem(hDlg, IDC_SOURCE_MEDIA_DESCRIPTOR_LABEL), lpData->fRewriteFat12);
	EnableWindow(GetDlgItem(hDlg, IDC_SOURCE_SECTORS_PER_TRACK_LABEL), lpData->fRewriteFat12);
	EnableWindow(GetDlgItem(hDlg, IDC_SOURCE_NUMBER_OF_HEADS_LABEL), lpData->fRewriteFat12);
	EnableWindow(GetDlgItem(hDlg, IDC_SOURCE_NUMBER_OF_SECTORS_VALUE), lpData->fRewriteFat12);
	EnableWindow(GetDlgItem(hDlg, IDC_SOURCE_MEDIA_DESCRIPTOR_VALUE), lpData->fRewriteFat12);
	EnableWindow(GetDlgItem(hDlg, IDC_SOURCE_SECTORS_PER_TRACK_VALUE), lpData->fRewriteFat12);
	EnableWindow(GetDlgItem(hDlg, IDC_SOURCE_NUMBER_OF_HEADS_VALUE), lpData->fRewriteFat12);
	EnableWindow(GetDlgItem(hDlg, IDC_TARGET_DRIVE_GROUPBOX), lpData->fRewriteFat12);
	EnableWindow(GetDlgItem(hDlg, IDC_TARGET_NUMBER_OF_SECTORS_LABEL), lpData->fRewriteFat12);
	EnableWindow(GetDlgItem(hDlg, IDC_TARGET_MEDIA_DESCRIPTOR_LABEL), lpData->fRewriteFat12);
	EnableWindow(GetDlgItem(hDlg, IDC_TARGET_SECTORS_PER_TRACK_LABEL), lpData->fRewriteFat12);
	EnableWindow(GetDlgItem(hDlg, IDC_TARGET_NUMBER_OF_HEADS_LABEL), lpData->fRewriteFat12);
	EnableWindow(GetDlgItem(hDlg, IDC_TARGET_NUMBER_OF_SECTORS_VALUE), lpData->fRewriteFat12);
	EnableWindow(GetDlgItem(hDlg, IDC_TARGET_MEDIA_DESCRIPTOR_VALUE), lpData->fRewriteFat12);
	EnableWindow(GetDlgItem(hDlg, IDC_TARGET_SECTORS_PER_TRACK_VALUE), lpData->fRewriteFat12);
	EnableWindow(GetDlgItem(hDlg, IDC_TARGET_NUMBER_OF_HEADS_VALUE), lpData->fRewriteFat12);

	char szValue[MAX_VALUE];
	sprintf(szValue, "%lu KB", lpData->dwSourceImageSize / 1024);
	SetWindowText(GetDlgItem(hDlg, IDC_SOURCE_IMAGE_SIZE), szValue);
	sprintf(szValue, "%lu KB", lpData->dwTargetDriveCapacity / 1024);
	SetWindowText(GetDlgItem(hDlg, IDC_TARGET_DRIVE_CAPACITY), szValue);
	sprintf(szValue, "%i", lpData->SourceInfo.wNumberOfSectors);
	SetWindowText(GetDlgItem(hDlg, IDC_SOURCE_NUMBER_OF_SECTORS_VALUE), szValue);
	sprintf(szValue, "0x%.2X", lpData->SourceInfo.bMediaDescriptor);
	SetWindowText(GetDlgItem(hDlg, IDC_SOURCE_MEDIA_DESCRIPTOR_VALUE), szValue);
	sprintf(szValue, "%i", lpData->SourceInfo.wSectorsPerTrack);
	SetWindowText(GetDlgItem(hDlg, IDC_SOURCE_SECTORS_PER_TRACK_VALUE), szValue);
	sprintf(szValue, "%i", lpData->SourceInfo.wNumberOfHeads);
	SetWindowText(GetDlgItem(hDlg, IDC_SOURCE_NUMBER_OF_HEADS_VALUE), szValue);
	sprintf(szValue, "%i", lpData->TargetInfo.wNumberOfSectors);
	SetWindowText(GetDlgItem(hDlg, IDC_TARGET_NUMBER_OF_SECTORS_VALUE), szValue);
	sprintf(szValue, "0x%.2X", lpData->TargetInfo.bMediaDescriptor);
	SetWindowText(GetDlgItem(hDlg, IDC_TARGET_MEDIA_DESCRIPTOR_VALUE), szValue);
	sprintf(szValue, "%i", lpData->TargetInfo.wSectorsPerTrack);
	SetWindowText(GetDlgItem(hDlg, IDC_TARGET_SECTORS_PER_TRACK_VALUE), szValue);
	sprintf(szValue, "%i", lpData->TargetInfo.wNumberOfHeads);
	SetWindowText(GetDlgItem(hDlg, IDC_TARGET_NUMBER_OF_HEADS_VALUE), szValue);
}

static void ConvertDlg_OnInitDialog(HWND hDlg, WPARAM wParam, LPARAM lParam)
{
	SetWindowLongPtr(hDlg, GWLP_USERDATA, lParam);
	struct ConvertDlgData *lpData = (struct ConvertDlgData*) lParam;

	CenterWindow(hDlg, GetParent(hDlg));
	UpdateControls(hDlg, lpData);
}

static void ConvertDlg_OnRewriteFat12(HWND hDlg, WPARAM wParam)
{
	struct ConvertDlgData *lpData = (struct ConvertDlgData*) GetWindowLongPtr(hDlg, GWLP_USERDATA);

	lpData->fRewriteFat12 = IsDlgButtonChecked(hDlg, IDC_REWRITE_FAT12) == BST_CHECKED;
	UpdateControls(hDlg, lpData);
}

static void ConvertDlg_OnOK(HWND hDlg, WPARAM wParam)
{
	EndDialog(hDlg, IDOK);
}

static void ConvertDlg_OnCancel(HWND hDlg, WPARAM wParam)
{
	SendMessage(hDlg, WM_CLOSE, 0, 0);
}

static void ConvertDlg_OnClose(HWND hDlg)
{
	EndDialog(hDlg, IDCANCEL);
}

static const CTRL_HANDLER ConvertDlgProc_Handler[] = {
	{ IDC_REWRITE_FAT12, ConvertDlg_OnRewriteFat12 },
	{ IDOK,              ConvertDlg_OnOK           },
	{ IDCANCEL,          ConvertDlg_OnCancel       },
};

INT_PTR CALLBACK ConvertDlgProc(HWND hDlg, UINT uMsg, WPARAM wParam, LPARAM lParam)
{
	BOOL fResult = FALSE;

	switch (uMsg)
	{
	case WM_INITDIALOG:
		// [...] return TRUE to direct the system to set the keyboard focus [...]
		ConvertDlg_OnInitDialog(hDlg, wParam, lParam);
		fResult = TRUE;
		break;
	case WM_COMMAND:
		// If an application processes this message, it should return zero.
		HandleControlMessage(hDlg, wParam, ConvertDlgProc_Handler, ARRAYSIZE(ConvertDlgProc_Handler));
		break;
	case WM_CLOSE:
		// If an application processes this message, it should return zero.
		ConvertDlg_OnClose(hDlg);
		break;
	}

	return fResult;
}
