#include <stdio.h>
#include <windows.h>

#include "debug.h"

#ifdef _DEBUG
#define TRACE_BUFFER_SIZE 250
void Trace(const char *s, ...)
{
	va_list args;
	char szBuffer[TRACE_BUFFER_SIZE];

	va_start(args, s);
	vsprintf(szBuffer, s, args);
	va_end(args);

#ifdef __MINGW32__
	fputs(szBuffer, stdout);
#else
	OutputDebugString(szBuffer);
#endif
}

void *DebugMalloc(size_t n, const char *file, int line)
{
	void *p = malloc(n);
	TRACE("MALLOC: %lu bytes at 0x%p from %s:%i\n", n, p, file, line);
	return p;
}

void *DebugRealloc(void *p, size_t n, const char *file, int line)
{
	void *q = realloc(p, n);
	TRACE("REALLOC: %lu bytes at 0x%p -> 0x%p from %s:%i\n", n, p, q, file, line);
	return q;
}

void DebugFree(void *p, const char *file, int line)
{
	TRACE("FREE: 0x%p from %s:%i\n", p, file, line);
	free(p);
}

#endif
