#include <stdio.h>
#include <stdlib.h>
#include <windows.h>

#include "settings.h"
#include "resource.h"
#include "guiutils.h"

#include "debug.h"

#include "delaysdlg.h"

#define MAX_VALUE 50

static void UpdateControls(HWND hDlg, const struct Settings *lpSettings)
{
	char szValue[MAX_VALUE];
	sprintf(szValue, "%i", lpSettings->wDelayBeforeWrite);
	SetWindowText(GetDlgItem(hDlg, IDC_DELAY_BEFORE_WRITE), szValue);
	sprintf(szValue, "%i", lpSettings->wDelayAfterWrite);
	SetWindowText(GetDlgItem(hDlg, IDC_DELAY_AFTER_WRITE), szValue);
	sprintf(szValue, "%i", lpSettings->wDelayBeforeRead);
	SetWindowText(GetDlgItem(hDlg, IDC_DELAY_BEFORE_READ), szValue);
	sprintf(szValue, "%i", lpSettings->wDelayAfterRead);
	SetWindowText(GetDlgItem(hDlg, IDC_DELAY_AFTER_READ), szValue);
	sprintf(szValue, "%i", lpSettings->wDelayBeforeReadAfterWrite);
	SetWindowText(GetDlgItem(hDlg, IDC_DELAY_BEFORE_READ_AFTER_WRITE), szValue);
	sprintf(szValue, "%i", lpSettings->wDelayBeforeWriteAfterRead);
	SetWindowText(GetDlgItem(hDlg, IDC_DELAY_BEFORE_WRITE_AFTER_READ), szValue);
	sprintf(szValue, "%i", lpSettings->wReadTimeout);
	SetWindowText(GetDlgItem(hDlg, IDC_READ_TIMEOUT), szValue);
}

static void DelaysDlg_OnInitDialog(HWND hDlg, WPARAM wParam, LPARAM lParam)
{
	SetWindowLongPtr(hDlg, GWLP_USERDATA, lParam);
	const struct Settings *lpSettings = (struct Settings*) lParam;

	CenterWindow(hDlg, GetParent(hDlg));
	UpdateControls(hDlg, lpSettings);
}

static void DelaysDlg_OnOK(HWND hDlg, WPARAM wParam)
{
	struct Settings *lpSettings = (struct Settings*) GetWindowLongPtr(hDlg, GWLP_USERDATA);

	char szValue[MAX_VALUE];

	GetWindowText(GetDlgItem(hDlg, IDC_DELAY_BEFORE_WRITE), szValue, sizeof(szValue));
	lpSettings->wDelayBeforeWrite = (WORD) strtoul(szValue, NULL, 0);
	GetWindowText(GetDlgItem(hDlg, IDC_DELAY_AFTER_WRITE), szValue, sizeof(szValue));
	lpSettings->wDelayAfterWrite = (WORD) strtoul(szValue, NULL, 0);
	GetWindowText(GetDlgItem(hDlg, IDC_DELAY_BEFORE_READ), szValue, sizeof(szValue));
	lpSettings->wDelayBeforeRead = (WORD) strtoul(szValue, NULL, 0);
	GetWindowText(GetDlgItem(hDlg, IDC_DELAY_AFTER_READ), szValue, sizeof(szValue));
	lpSettings->wDelayAfterRead = (WORD) strtoul(szValue, NULL, 0);
	GetWindowText(GetDlgItem(hDlg, IDC_DELAY_BEFORE_READ_AFTER_WRITE), szValue, sizeof(szValue));
	lpSettings->wDelayBeforeReadAfterWrite = (WORD) strtoul(szValue, NULL, 0);
	GetWindowText(GetDlgItem(hDlg, IDC_DELAY_BEFORE_WRITE_AFTER_READ), szValue, sizeof(szValue));
	lpSettings->wDelayBeforeWriteAfterRead = (WORD) strtoul(szValue, NULL, 0);
	GetWindowText(GetDlgItem(hDlg, IDC_READ_TIMEOUT), szValue, sizeof(szValue));
	lpSettings->wReadTimeout = (WORD) strtoul(szValue, NULL, 0);

	EndDialog(hDlg, IDOK);
}

static void DelaysDlg_OnLoadDefaults(HWND hDlg, WPARAM wParam)
{
	struct Settings defaults;
	LoadDefaultSettings(&defaults, SETTINGS_DELAYS);
	UpdateControls(hDlg, &defaults);
}

static void DelaysDlg_OnCancel(HWND hDlg, WPARAM wParam)
{
	SendMessage(hDlg, WM_CLOSE, 0, 0);
}

static void DelaysDlg_OnClose(HWND hDlg)
{
	EndDialog(hDlg, IDCANCEL);
}

static const CTRL_HANDLER DelaysDlgProc_Handler[] = {
	{ IDOK,               DelaysDlg_OnOK     },
	{ IDC_LOAD_DEFAULTS,  DelaysDlg_OnLoadDefaults },
	{ IDCANCEL,           DelaysDlg_OnCancel },
};

INT_PTR CALLBACK DelaysDlgProc(HWND hDlg, UINT uMsg, WPARAM wParam, LPARAM lParam)
{
	BOOL fResult = FALSE;

	switch (uMsg)
	{
	case WM_INITDIALOG:
		// [...] return TRUE to direct the system to set the keyboard focus [...]
		DelaysDlg_OnInitDialog(hDlg, wParam, lParam);
		fResult = TRUE;
		break;
	case WM_COMMAND:
		// If an application processes this message, it should return zero.
		HandleControlMessage(hDlg, wParam, DelaysDlgProc_Handler, ARRAYSIZE(DelaysDlgProc_Handler));
		break;
	case WM_CLOSE:
		// If an application processes this message, it should return zero.
		DelaysDlg_OnClose(hDlg);
		break;
	}

	return fResult;
}
