#include <stdio.h>
#include <windows.h>

#include "resource.h"
#include "guiutils.h"

#include "debug.h"

#include "driveeditentrydlg.h"
#include "driveeditdlg.h"

static void UpdateControlState(HWND hDlg)
{
	const int iCount = (int) SendMessage(GetDlgItem(hDlg, IDC_DRIVELIST), CB_GETCOUNT, 0, 0);

	EnableWindow(GetDlgItem(hDlg, IDC_DRIVELIST), iCount > 0);
	EnableWindow(GetDlgItem(hDlg, IDC_ADD), iCount < MAX_DRIVES);
	EnableWindow(GetDlgItem(hDlg, IDC_EDIT), iCount > 0);
	EnableWindow(GetDlgItem(hDlg, IDC_REMOVE), iCount > 0);
}

// Code duplication with maindlg.c
static void UpdateDriveInfo(HWND hDlg)
{
	HWND hItem = GetDlgItem(hDlg, IDC_DRIVELIST);
	const int iSel = (int) SendMessage(hItem, CB_GETCURSEL, 0, 0);
	if (iSel != CB_ERR)
	{
		struct DriveListEntry *lpEntry = (struct DriveListEntry*) SendMessage(hItem, CB_GETITEMDATA, iSel, 0);
		ASSERT(lpEntry != NULL);
		if (lpEntry)
		{
			SetWindowText(GetDlgItem(hDlg, IDC_TYPE), GetDriveTypeString(lpEntry->Info.bDriveType));
			char szText[MAX_LABEL];
			sprintf(szText, "%i", lpEntry->Info.bNumberOfTracks);
			SetWindowText(GetDlgItem(hDlg, IDC_TRACKS), szText);
			sprintf(szText, "%i", lpEntry->Info.bNumberOfSides);
			SetWindowText(GetDlgItem(hDlg, IDC_SIDES), szText);
			sprintf(szText, "%i", lpEntry->Info.bSectorsPerTrack);
			SetWindowText(GetDlgItem(hDlg, IDC_SECTORS), szText);
		}
	}
	else
	{
		SetWindowText(GetDlgItem(hDlg, IDC_TYPE), "-");
		SetWindowText(GetDlgItem(hDlg, IDC_TRACKS), "-");
		SetWindowText(GetDlgItem(hDlg, IDC_SIDES), "-");
		SetWindowText(GetDlgItem(hDlg, IDC_SECTORS), "-");
	}
}

static void InsertDriveIntoList(HWND hDriveList, int iPosition, const struct DriveListEntry *lpEntry)
{
	char szName[MAX_DRIVE_NAME];
	sprintf(szName, "Drive %i", lpEntry->bDriveIndex);
	int iItemIndex = (int) SendMessage(hDriveList, CB_INSERTSTRING, iPosition, (LPARAM) szName);
	ASSERT(iItemIndex != CB_ERR);
	SendMessage(hDriveList, CB_SETITEMDATA, iItemIndex, (LPARAM) lpEntry);
	SendMessage(hDriveList, CB_SETCURSEL, iItemIndex, 0);
}

static void DriveEditDlg_OnInitDialog(HWND hDlg, WPARAM wParam, LPARAM lParam)
{
	SetWindowLongPtr(hDlg, GWLP_USERDATA, lParam);
	struct DriveEditDlgData *lpData = (struct DriveEditDlgData*) lParam;

	CenterWindow(hDlg, GetParent(hDlg));

	HWND hDriveList = GetDlgItem(hDlg, IDC_DRIVELIST);
	for (int i = 0; i < ARRAYSIZE(lpData->lpEntries); i++)
	{
		struct DriveListEntry *lpEntry = lpData->lpEntries[i];

		if (lpEntry)
		{
			InsertDriveIntoList(hDriveList, -1, lpEntry);
			// Memory now belongs to the list control
			lpData->lpEntries[i] = NULL;
		}
	}
	SendMessage(hDriveList, CB_SETCURSEL, 0, 0);
	UpdateDriveInfo(hDlg);
	UpdateControlState(hDlg);
}

static void DriveEditDlg_OnDriveList(HWND hDlg, WPARAM wParam)
{
	if (HIWORD(wParam) == CBN_SELCHANGE)
		UpdateDriveInfo(hDlg);
}

static void DriveEditDlg_OnAdd(HWND hDlg, WPARAM wParam)
{
	struct DriveEditEntryDlgData DlgData = { DRIVEEDIT_ADD };

	HINSTANCE hInst = (HINSTANCE) GetWindowLongPtr(hDlg, GWLP_HINSTANCE);
	LPCSTR lpTemplate = MAKEINTRESOURCE(IDD_DRIVEEDITENTRYDLG);
	INT_PTR iResult = DialogBoxParam(hInst, lpTemplate, hDlg, DriveEditEntryDlgProc, (LPARAM) &DlgData);
	if (iResult == IDOK)
	{
		struct DriveListEntry *lpEntry = MALLOC(sizeof(*lpEntry));
		if (lpEntry)
		{
			*lpEntry = DlgData.Entry;
			InsertDriveIntoList(GetDlgItem(hDlg, IDC_DRIVELIST), -1, lpEntry);
			UpdateControlState(hDlg);
			UpdateDriveInfo(hDlg);
		}
		else
		{
			FormatMessageBox(hDlg, MB_ICONERROR, "Cannot store drive info, out of memory?");
		}
	}
}

static void DriveEditDlg_OnEdit(HWND hDlg, WPARAM wParam)
{
	struct DriveEditEntryDlgData DlgData = { DRIVEEDIT_EDIT };
	struct DriveListEntry *lpEntry = NULL;

	HWND hDriveList = GetDlgItem(hDlg, IDC_DRIVELIST);
	int iSel = (int) SendMessage(hDriveList, CB_GETCURSEL, 0, 0);
	if (iSel != CB_ERR)
	{
		lpEntry = (struct DriveListEntry*) SendMessage(hDriveList, CB_GETITEMDATA, iSel, 0);
		ASSERT(lpEntry != NULL);
	}

	if (lpEntry)
	{
		DlgData.Entry = *lpEntry;

		HINSTANCE hInst = (HINSTANCE) GetWindowLongPtr(hDlg, GWLP_HINSTANCE);
		LPCSTR lpTemplate = MAKEINTRESOURCE(IDD_DRIVEEDITENTRYDLG);
		INT_PTR iResult = DialogBoxParam(hInst, lpTemplate, hDlg, DriveEditEntryDlgProc, (LPARAM) &DlgData);
		if (iResult == IDOK)
		{
			*lpEntry = DlgData.Entry;

			// Drive index might have changed, so item has to be replaced
			SendMessage(hDriveList, CB_DELETESTRING, iSel, 0);
			InsertDriveIntoList(hDriveList, iSel, lpEntry);
			UpdateControlState(hDlg);
			UpdateDriveInfo(hDlg);
		}
	}
}

static void DriveEditDlg_OnRemove(HWND hDlg, WPARAM wParam)
{
	HWND hItem = GetDlgItem(hDlg, IDC_DRIVELIST);
	int iSel = (int) SendMessage(hItem, CB_GETCURSEL, 0, 0);
	if (iSel != CB_ERR)
	{
		struct DriveListEntry *lpEntry = (struct DriveListEntry*) SendMessage(hItem, CB_GETITEMDATA, iSel, 0);
		ASSERT(lpEntry != NULL);

		FREE(lpEntry);
		SendMessage(hItem, CB_DELETESTRING, iSel, 0);
		SendMessage(GetDlgItem(hDlg, IDC_DRIVELIST), CB_SETCURSEL, max(iSel - 1, 0), 0);
		UpdateControlState(hDlg);
		UpdateDriveInfo(hDlg);
	}
}

static void DriveEditDlg_OnOK(HWND hDlg, WPARAM wParam)
{
	struct DriveEditDlgData *lpData = (struct DriveEditDlgData*) GetWindowLongPtr(hDlg, GWLP_USERDATA);

	HWND hDriveList = GetDlgItem(hDlg, IDC_DRIVELIST);
	const int iCount = (int) SendMessage(hDriveList, CB_GETCOUNT, 0, 0);
	for (int iItemIndex = 0; iItemIndex < iCount; iItemIndex++)
	{
		if (iItemIndex < ARRAYSIZE(lpData->lpEntries))
		{
			void *lpMem = (void*) SendMessage(hDriveList, CB_GETITEMDATA, iItemIndex, 0);
			lpData->lpEntries[iItemIndex] = lpMem;
			SendMessage(hDriveList, CB_SETITEMDATA, iItemIndex, 0);
		}
	}

	EndDialog(hDlg, IDOK);
}

static void DriveEditDlg_OnCancel(HWND hDlg, WPARAM wParam)
{
	SendMessage(hDlg, WM_CLOSE, 0, 0);
}

static void DriveEditDlg_OnClose(HWND hDlg)
{
	EndDialog(hDlg, IDCANCEL);
}

static void DriveEditDlg_OnDestroy(HWND hDlg)
{
	HWND hItem = GetDlgItem(hDlg, IDC_DRIVELIST);
	const int iCount = (int) SendMessage(hItem, CB_GETCOUNT, 0, 0);
	for (int iEntryIndex = 0; iEntryIndex < iCount; iEntryIndex++)
	{
		void *lpMem = (void*) SendMessage(hItem, CB_GETITEMDATA, iEntryIndex, 0);
		if (lpMem)
			FREE(lpMem);
	}
}

static const CTRL_HANDLER DriveEditDlgProc_Handler[] = {
	{ IDC_DRIVELIST,  DriveEditDlg_OnDriveList },
	{ IDC_ADD,        DriveEditDlg_OnAdd       },
	{ IDC_EDIT,       DriveEditDlg_OnEdit      },
	{ IDC_REMOVE,     DriveEditDlg_OnRemove    },
	{ IDOK,           DriveEditDlg_OnOK        },
	{ IDCANCEL,       DriveEditDlg_OnCancel    },
};

INT_PTR CALLBACK DriveEditDlgProc(HWND hDlg, UINT uMsg, WPARAM wParam, LPARAM lParam)
{
	BOOL fResult = FALSE;

	switch (uMsg)
	{
	case WM_INITDIALOG:
		// [...] return TRUE to direct the system to set the keyboard focus [...]
		DriveEditDlg_OnInitDialog(hDlg, wParam, lParam);
		fResult = TRUE;
		break;
	case WM_COMMAND:
		// If an application processes this message, it should return zero.
		HandleControlMessage(hDlg, wParam, DriveEditDlgProc_Handler, ARRAYSIZE(DriveEditDlgProc_Handler));
		break;
	case WM_CLOSE:
		// If an application processes this message, it should return zero.
		DriveEditDlg_OnClose(hDlg);
		break;
	case WM_DESTROY:
		DriveEditDlg_OnDestroy(hDlg);
		break;
	}

	return fResult;
}
