#ifndef DRIVEEDITDLG_H
#define DRIVEEDITDLG_H

#include <windows.h>

#include "drivelist.h"

// Rules for memory ownership
// 1. Caller creates duplicates of master list entries and places them in list
// 2. Dialog transfers entry ownership to list control, sets list slots to NULL
// 3. User works with list control
//    a. Adding new entries allocates memory
//    b. Removing entries frees memory
// 4. Selecting Cancel closes dialog without extra work here
// 5. Selecting OK transfers entry ownership back to list, sets control slots to NULL
// 6. Destroying dialog frees memory from control slots (robustness for cancel/close)
// 7. If left with OK, caller updates master list
//    a. Clear list (freeing previous entries)
//    b. Move entries from list slots to list, set slots to NULL
// 8. Caller frees memory from list slots that are not  NULL
//
// Even though this mechanism should be robust against a list control with more entries
// than list slots available, adding entries beyond MAX_DRIVES should be prevented to
// avoid unpleasant user experience due to discarding items when the editor is closed.

struct DriveEditDlgData {
	struct DriveListEntry *lpEntries[MAX_DRIVES];
};

INT_PTR CALLBACK DriveEditDlgProc(HWND hDlg, UINT uMsg, WPARAM wParam, LPARAM lParam);

#endif
