#include <stdio.h>
#include <windows.h>

#include "resource.h"
#include "guiutils.h"

#include "debug.h"

#include "driveeditentrydlg.h"

static void InitDriveTypeList(HWND hTypeList)
{
	for (BYTE bType = DRIVE_TYPE_FIRST; bType <= DRIVE_TYPE_LAST; bType++)
	{
		int iItemIndex = (int) SendMessage(hTypeList, CB_ADDSTRING, 0, (LPARAM) GetDriveTypeString(bType));
		ASSERT(iItemIndex != CB_ERR);
		SendMessage(hTypeList, CB_SETITEMDATA, iItemIndex, bType);
	}
	SendMessage(hTypeList, CB_SETCURSEL, 0, 0);
}

static void SetDriveTypeListEntry(HWND hTypeList, BYTE bType)
{
	const int iCount = (int) SendMessage(hTypeList, CB_GETCOUNT, 0, 0);

	for (int iIndex = 0; iIndex < iCount; iIndex++)
	{
		if (SendMessage(hTypeList, CB_GETITEMDATA, iIndex, 0) == bType)
		{
			SendMessage(hTypeList, CB_SETCURSEL, iIndex, 0);
			break;
		}
	}
}

static BYTE GetDriveTypeListEntry(HWND hTypeList)
{
	BOOL bType = 0x00;

	const int iSel = (int) SendMessage(hTypeList, CB_GETCURSEL, 0, 0);
	if (iSel != CB_ERR)
	{
		bType = (BYTE) SendMessage(hTypeList, CB_GETITEMDATA, iSel, 0);
	}

	return bType;
}

static void LoadControlValues(HWND hDlg, const struct DriveListEntry *lpEntry)
{
	char szBuffer[MAX_LABEL];

	sprintf(szBuffer, "%i", lpEntry->bDriveIndex);
	SetWindowText(GetDlgItem(hDlg, IDC_INDEX), szBuffer);
	SetDriveTypeListEntry(GetDlgItem(hDlg, IDC_TYPE), lpEntry->Info.bDriveType);
	sprintf(szBuffer, "%i", lpEntry->Info.bNumberOfTracks);
	SetWindowText(GetDlgItem(hDlg, IDC_TRACKS), szBuffer);
	sprintf(szBuffer, "%i", lpEntry->Info.bNumberOfSides);
	SetWindowText(GetDlgItem(hDlg, IDC_SIDES), szBuffer);
	sprintf(szBuffer, "%i", lpEntry->Info.bSectorsPerTrack);
	SetWindowText(GetDlgItem(hDlg, IDC_SECTORS), szBuffer);
}

static void StoreControlValues(HWND hDlg, struct DriveListEntry *lpEntry)
{
	char szBuffer[MAX_LABEL];

	GetWindowText(GetDlgItem(hDlg, IDC_INDEX), szBuffer, sizeof(szBuffer));
	lpEntry->bDriveIndex = (BYTE) strtoul(szBuffer, NULL, 0);
	lpEntry->Info.bDriveType = GetDriveTypeListEntry(GetDlgItem(hDlg, IDC_TYPE));
	GetWindowText(GetDlgItem(hDlg, IDC_TRACKS), szBuffer, sizeof(szBuffer));
	lpEntry->Info.bNumberOfTracks = (BYTE) strtoul(szBuffer, NULL, 0);
	GetWindowText(GetDlgItem(hDlg, IDC_SIDES), szBuffer, sizeof(szBuffer));
	lpEntry->Info.bNumberOfSides = (BYTE) strtoul(szBuffer, NULL, 0);
	GetWindowText(GetDlgItem(hDlg, IDC_SECTORS), szBuffer, sizeof(szBuffer));
	lpEntry->Info.bSectorsPerTrack = (BYTE) strtoul(szBuffer, NULL, 0);
}

static void DriveEditEntryDlg_OnInitDialog(HWND hDlg, WPARAM wParam, LPARAM lParam)
{
	SetWindowLongPtr(hDlg, GWLP_USERDATA, lParam);
	struct DriveEditEntryDlgData *lpData = (struct DriveEditEntryDlgData*) lParam;

	InitDriveTypeList(GetDlgItem(hDlg, IDC_TYPE));

	if (lpData->bMode == DRIVEEDIT_ADD)
	{
		SetWindowText(hDlg, "Add drive");
	}
	else if (lpData->bMode == DRIVEEDIT_EDIT)
	{
		SetWindowText(hDlg, "Edit drive");
		LoadControlValues(hDlg, &lpData->Entry);
	}
	else
	{
		// Invalid editor mode
		ASSERT(FALSE);
	}
}

static void DriveEditEntryDlg_OnOK(HWND hDlg, WPARAM wParam)
{
	struct DriveEditEntryDlgData *lpData = (struct DriveEditEntryDlgData*) GetWindowLongPtr(hDlg, GWLP_USERDATA);

	StoreControlValues(hDlg, &lpData->Entry);

	EndDialog(hDlg, IDOK);
}

static void DriveEditEntryDlg_OnCancel(HWND hDlg, WPARAM wParam)
{
	SendMessage(hDlg, WM_CLOSE, 0, 0);
}

static void DriveEditEntryDlg_OnClose(HWND hDlg)
{
	EndDialog(hDlg, IDCANCEL);
}

static const CTRL_HANDLER DriveEditEntryDlgProc_Handler[] = {
	{ IDOK,           DriveEditEntryDlg_OnOK     },
	{ IDCANCEL,       DriveEditEntryDlg_OnCancel },
};

INT_PTR CALLBACK DriveEditEntryDlgProc(HWND hDlg, UINT uMsg, WPARAM wParam, LPARAM lParam)
{
	BOOL fResult = FALSE;

	switch (uMsg)
	{
	case WM_INITDIALOG:
		// [...] return TRUE to direct the system to set the keyboard focus [...]
		DriveEditEntryDlg_OnInitDialog(hDlg, wParam, lParam);
		fResult = TRUE;
		break;
	case WM_COMMAND:
		// If an application processes this message, it should return zero.
		HandleControlMessage(hDlg, wParam, DriveEditEntryDlgProc_Handler, ARRAYSIZE(DriveEditEntryDlgProc_Handler));
		break;
	case WM_CLOSE:
		// If an application processes this message, it should return zero.
		DriveEditEntryDlg_OnClose(hDlg);
		break;
	}

	return fResult;
}
