#include "debug.h"
#include "driveutils.h"

static const LPCSTR DriveTypeStrings[] = { "undef", "360K", "1.2M", "720K", "1.44M" };

LPCSTR GetDriveTypeString(BYTE bDriveType)
{
	return DriveTypeStrings[(bDriveType < ARRAYSIZE(DriveTypeStrings)) ? bDriveType : 0];
}

WORD GetTotalSectorCount(const struct DriveInfo *lpDriveInfo)
{
	ASSERT(lpDriveInfo != NULL);

	return lpDriveInfo->bNumberOfTracks * lpDriveInfo->bNumberOfSides * lpDriveInfo->bSectorsPerTrack;
}

BOOL SectorIndexToDrivePos(WORD wSectorIndex, const struct DriveInfo *lpDriveInfo, struct DrivePosition *lpTargetPos)
{
	ASSERT(lpDriveInfo != NULL);
	ASSERT(lpTargetPos != NULL);

	BOOL fResult = FALSE;

	if (wSectorIndex < GetTotalSectorCount(lpDriveInfo))
	{
		lpTargetPos->bTrack = wSectorIndex / (lpDriveInfo->bNumberOfSides * lpDriveInfo->bSectorsPerTrack);
		wSectorIndex -= lpTargetPos->bTrack * lpDriveInfo->bNumberOfSides * lpDriveInfo->bSectorsPerTrack;

		lpTargetPos->bSide = wSectorIndex / lpDriveInfo->bSectorsPerTrack;
		wSectorIndex -= lpTargetPos->bSide * lpDriveInfo->bSectorsPerTrack;

		lpTargetPos->bSector = (BYTE) wSectorIndex;

		fResult = TRUE;
	}

	return fResult;
}
