#include <stdio.h>
#include <string.h>
#include <windows.h>
#include <commctrl.h>
#include <shellapi.h>
#include <ctype.h>

#include "resource.h"
#include "guiutils.h"
#include "list.h"

#include "debug.h"

#include "pickfiledlg.h"
#include "transferfile.h"
#include "findfile.h"
#include "exploredlg.h"

#define COL_NAME 0
#define COL_SIZE 1
#define COL_DATE 2
#define COL_TIME 3
#define COL_ATTR 4

static void ExploreDlg_OnInitDialog(HWND hDlg, WPARAM wParam, LPARAM lParam)
{
	SetWindowLongPtr(hDlg, GWLP_USERDATA, lParam);
	struct ExploreDlgData *lpData = (struct ExploreDlgData*) lParam;

	CenterWindow(hDlg, GetParent(hDlg));
	lpData->hListCtrl = CreateListControl(hDlg, GetDlgItem(hDlg, IDC_LISTCTRL_DUMMY));

	if (lpData->hListCtrl)
	{
		LVCOLUMN col;
		col.mask = LVCF_TEXT;
		col.pszText = "Name";
		SendMessage(lpData->hListCtrl, LVM_INSERTCOLUMN, COL_NAME, (LPARAM) &col);
		col.pszText = "Size";
		SendMessage(lpData->hListCtrl, LVM_INSERTCOLUMN, COL_SIZE, (LPARAM) &col);
		col.pszText = "Date";
		SendMessage(lpData->hListCtrl, LVM_INSERTCOLUMN, COL_DATE, (LPARAM) &col);
		col.pszText = "Time";
		SendMessage(lpData->hListCtrl, LVM_INSERTCOLUMN, COL_TIME, (LPARAM) &col);
		col.pszText = "Attr";
		SendMessage(lpData->hListCtrl, LVM_INSERTCOLUMN, COL_ATTR, (LPARAM) &col);

		SendMessage(lpData->hListCtrl, LVM_SETCOLUMNWIDTH, COL_NAME, LVSCW_AUTOSIZE_USEHEADER);
		SendMessage(lpData->hListCtrl, LVM_SETCOLUMNWIDTH, COL_SIZE, LVSCW_AUTOSIZE_USEHEADER);
		SendMessage(lpData->hListCtrl, LVM_SETCOLUMNWIDTH, COL_DATE, LVSCW_AUTOSIZE_USEHEADER);
		SendMessage(lpData->hListCtrl, LVM_SETCOLUMNWIDTH, COL_TIME, LVSCW_AUTOSIZE_USEHEADER);
		SendMessage(lpData->hListCtrl, LVM_SETCOLUMNWIDTH, COL_ATTR, LVSCW_AUTOSIZE_USEHEADER);
	}
}

// TODO: find better function names
static void AddFileListItem(HWND hListCtrl, int iPos, const struct FindResult *lpItem)
{
	char szBuffer[100];
	LVITEM item = { 0 };

	item.iItem = iPos;

	item.iSubItem = COL_NAME;
	item.pszText = (LPSTR) lpItem->szFileName;
	item.lParam = lpItem->bAttributes;
	item.mask = LVIF_TEXT | LVIF_PARAM;
	SendMessage(hListCtrl, LVM_INSERTITEM, 0, (LPARAM) &item);

	item.mask = LVIF_TEXT;
	item.iSubItem = COL_SIZE;
	sprintf(szBuffer, "%lu", lpItem->dwFileSize);
	item.pszText = (lpItem->bAttributes & FILE_ATTR_SUBDIR) ? "<DIR>" : szBuffer;
	SendMessage(hListCtrl, LVM_SETITEM, 0, (LPARAM) &item);

	item.iSubItem = COL_DATE;
	sprintf(szBuffer, "%.2i.%.2i.%i", lpItem->FileDate.bDay, lpItem->FileDate.bMonth, lpItem->FileDate.wYear);
	item.pszText = szBuffer;
	SendMessage(hListCtrl, LVM_SETITEM, 0, (LPARAM) &item);

	item.iSubItem = COL_TIME;
	sprintf(szBuffer, "%.2i:%.2i:%.2i", lpItem->FileTime.bHours, lpItem->FileTime.bMinutes, lpItem->FileTime.bSeconds);
	item.pszText = szBuffer;
	SendMessage(hListCtrl, LVM_SETITEM, 0, (LPARAM) &item);

	item.iSubItem = COL_ATTR;
	sprintf(szBuffer, "%c%c%c%c",
		lpItem->bAttributes & FILE_ATTR_READONLY ? 'R' : '-',
		lpItem->bAttributes & FILE_ATTR_ARCHIVE ? 'A' : '-',
		lpItem->bAttributes & FILE_ATTR_SYSTEM ? 'S' : '-',
		lpItem->bAttributes & FILE_ATTR_HIDDEN ? 'H' : '-');
	item.pszText = szBuffer;
	SendMessage(hListCtrl, LVM_SETITEM, 0, (LPARAM) &item);
}

static int DefaultSort(const void *lpLeft, const void *lpRight)
{
	const struct FindResult *lpLeftItem = *(const struct FindResult**) lpLeft;
	const struct FindResult *lpRightItem = *(const struct FindResult**) lpRight;

	BOOL fLeftDir = (lpLeftItem->bAttributes & FILE_ATTR_SUBDIR) != 0;
	BOOL fRightDir = (lpRightItem->bAttributes & FILE_ATTR_SUBDIR) != 0;

	if (fLeftDir && !fRightDir) return -1;
	if (!fLeftDir && fRightDir) return +1;

	const char *lpLeftChar = (const char*) lpLeftItem->szFileName;
	const char *lpRightChar = (const char*) lpRightItem->szFileName;
	while (*lpLeftChar && *lpRightChar && tolower(*lpLeftChar) == tolower(*lpRightChar))
	{
		lpLeftChar++;
		lpRightChar++;
	}
	return tolower(*lpLeftChar) - tolower(*lpRightChar);
}

static void GetDirectoryPart(LPCSTR lpPath, LPSTR lpDirectory, int iMaxDirectory)
{
	const char *lpLastSeparator = strrchr(lpPath, '\\');
	int iLength = (int) (lpLastSeparator ? (lpLastSeparator - lpPath) : 0);

	iLength = min(iLength, iMaxDirectory - 1);
	memcpy(lpDirectory, lpPath, iLength);
	lpDirectory[iLength] = '\0';
}

static void GetFilePart(LPCSTR lpPath, LPSTR lpFile, int iMaxFile)
{
	const char *lpLastSeparator = strrchr(lpPath, '\\');
	int iLength = (int) (lpLastSeparator ? strlen(lpLastSeparator + 1) : 0);

	iLength = min(iLength, iMaxFile - 1);
	memcpy(lpFile, lpLastSeparator + 1, iLength);
	lpFile[iLength] = '\0';
}

static void GoToNewLocation(HWND hDlg, LPCSTR lpPath)
{
	struct ExploreDlgData *lpData = (struct ExploreDlgData*) GetWindowLongPtr(hDlg, GWLP_USERDATA);
	GetDirectoryPart(lpPath, lpData->szCurrDirectory, sizeof(lpData->szCurrDirectory));

	SendMessage(lpData->hListCtrl, LVM_DELETEALLITEMS, 0, 0);
	DragAcceptFiles(hDlg, FALSE);

	struct List *lpFileList = CreateList();
	if (GetFileList(hDlg, lpData->lpDevice, lpPath, lpFileList))
	{
		const int iCount = GetListItemCount(lpFileList);

		if (iCount > 0)
		{
			SortList(lpFileList, DefaultSort);

			int iInsertPos = 0;
			SendMessage(lpData->hListCtrl, WM_SETREDRAW, FALSE, 0);
			for (int iIndex = 0; iIndex < iCount; iIndex++)
			{
				const struct FindResult *lpItem = GetListItem(lpFileList, iIndex);
				// apply any filtering here
				AddFileListItem(lpData->hListCtrl, iInsertPos++, lpItem);
			}
			SendMessage(lpData->hListCtrl, WM_SETREDRAW, TRUE, 0);

			SendMessage(lpData->hListCtrl, LVM_SETCOLUMNWIDTH, COL_NAME, LVSCW_AUTOSIZE_USEHEADER);
			SendMessage(lpData->hListCtrl, LVM_SETCOLUMNWIDTH, COL_SIZE, LVSCW_AUTOSIZE_USEHEADER);
			SendMessage(lpData->hListCtrl, LVM_SETCOLUMNWIDTH, COL_DATE, LVSCW_AUTOSIZE_USEHEADER);
			SendMessage(lpData->hListCtrl, LVM_SETCOLUMNWIDTH, COL_TIME, LVSCW_AUTOSIZE_USEHEADER);
			SendMessage(lpData->hListCtrl, LVM_SETCOLUMNWIDTH, COL_ATTR, LVSCW_AUTOSIZE_USEHEADER);

			DragAcceptFiles(hDlg, TRUE);
		}
	}
	else
	{
		FormatMessageBox(hDlg, MB_ICONERROR, "Get file list failed.");
	}
	DestroyList(lpFileList);

	// Invalidating list control only doesn't show scroll bars
	InvalidateRect(hDlg, NULL, TRUE);
}

static void QuerySendFile(HWND hDlg, LPCSTR lpPath)
{
	struct ExploreDlgData *lpData = (struct ExploreDlgData*) GetWindowLongPtr(hDlg, GWLP_USERDATA);
	HINSTANCE hInst = (HINSTANCE) GetWindowLongPtr(hDlg, GWLP_HINSTANCE);

	LPCSTR lpTemplate = MAKEINTRESOURCE(IDD_FILEPICKDLG);
	struct PickFileDlgParam DlgParam = { 0 };
	DlgParam.bMode = PFD_MODE_SEND;
	DlgParam.bFocus = PFD_FOCUS_REMOTE;
	strncpy(DlgParam.szLocalPath, lpPath, sizeof(DlgParam.szLocalPath));
	DlgParam.szLocalPath[sizeof(DlgParam.szLocalPath) - 1] = '\0';

	char szFileName[MAX_PATH];
	GetFilePart(lpPath, szFileName, sizeof(szFileName));
	char szRemotePath[MAX_PATH * 2];
	sprintf(szRemotePath, "%s\\%s", lpData->szCurrDirectory, szFileName);
	szRemotePath[MAX_PATH - 1] = '\0';
	strncpy(DlgParam.szRemotePath, szRemotePath, sizeof(DlgParam.szRemotePath));
	DlgParam.szRemotePath[sizeof(DlgParam.szRemotePath) - 1] = '\0';

	INT_PTR iResult = DialogBoxParam(hInst, lpTemplate, hDlg, PickFileDlgProc, (LPARAM) &DlgParam);
	if (iResult == IDOK)
	{
		if (SendFile(hDlg, lpData->lpDevice, DlgParam.szLocalPath, DlgParam.szRemotePath))
		{
			FormatMessageBox(hDlg, MB_ICONINFORMATION, "Send file succeeded.");
		}
		else
		{
			FormatMessageBox(hDlg, MB_ICONERROR, "Send file failed.");
		}
	}
}

static void QueryRecvFile(HWND hDlg, LPCSTR lpPath)
{
	struct ExploreDlgData *lpData = (struct ExploreDlgData*) GetWindowLongPtr(hDlg, GWLP_USERDATA);
	HINSTANCE hInst = (HINSTANCE) GetWindowLongPtr(hDlg, GWLP_HINSTANCE);

	LPCSTR lpTemplate = MAKEINTRESOURCE(IDD_FILEPICKDLG);
	struct PickFileDlgParam DlgParam = { 0 };
	DlgParam.bMode = PFD_MODE_RECEIVE;
	DlgParam.bFocus = PFD_FOCUS_LOCAL;
	strncpy(DlgParam.szRemotePath, lpPath, sizeof(DlgParam.szRemotePath));
	DlgParam.szRemotePath[sizeof(DlgParam.szRemotePath) - 1] = '\0';
	INT_PTR iResult = DialogBoxParam(hInst, lpTemplate, hDlg, PickFileDlgProc, (LPARAM) &DlgParam);
	if (iResult == IDOK)
	{
		if (RecvFile(hDlg, lpData->lpDevice, DlgParam.szRemotePath, DlgParam.szLocalPath))
		{
			FormatMessageBox(hDlg, MB_ICONINFORMATION, "Receive file succeeded.");
		}
		else
		{
			FormatMessageBox(hDlg, MB_ICONERROR, "Receive file failed.");
		}
	}
}

static void ExploreDlg_OnListCtrl(HWND hDlg, LPNMHDR lpNmHdr)
{
	struct ExploreDlgData *lpData = (struct ExploreDlgData*) GetWindowLongPtr(hDlg, GWLP_USERDATA);

	if (lpNmHdr->code == NM_DBLCLK)
	{
		LPNMITEMACTIVATE lpNmItemActivate = (LPNMITEMACTIVATE) lpNmHdr;

		// Do nothing if user clicked into empty space of list control
		if (lpNmItemActivate->iItem == -1)
			return;

		// Get DOS name of clicked entry
		char szName[MAX_DOS_FILENAME];
		LVITEM item = { 0 };
		item.iItem = lpNmItemActivate->iItem;
		item.mask = LVIF_TEXT | LVIF_PARAM;
		item.iSubItem = COL_NAME;
		item.pszText = szName;
		item.cchTextMax = sizeof(szName);
		SendMessage(lpData->hListCtrl, LVM_GETITEM, 0, (LPARAM) &item);

		if (item.lParam & FILE_ATTR_SUBDIR)
		{
			char szNewLocationRaw[MAX_PATH * 2];
			sprintf(szNewLocationRaw, "%s\\%s\\*.*", lpData->szCurrDirectory, szName);
			szNewLocationRaw[MAX_PATH - 1] = '\0';

			// Clean path from things like "C:\subdir\..\*.*"
			char szNewLocationCleaned[MAX_PATH];
			GetFullPathName(szNewLocationRaw, sizeof(szNewLocationCleaned), szNewLocationCleaned, NULL);

			SetWindowText(GetDlgItem(hDlg, IDC_LOCATION), szNewLocationCleaned);
			GoToNewLocation(hDlg, szNewLocationCleaned);
		}
		else
		{
			char szPath[MAX_PATH * 2];
			sprintf(szPath, "%s\\%s", lpData->szCurrDirectory, szName);
			szPath[MAX_PATH - 1] = '\0';

			QueryRecvFile(hDlg, szPath);
		}
	}
}

static void ExploreDlg_OnOK(HWND hDlg, WPARAM wParam)
{
	if (GetFocus() == GetDlgItem(hDlg, IDC_LOCATION))
	{
		char szPath[MAX_PATH];
		GetWindowText(GetDlgItem(hDlg, IDC_LOCATION), szPath, sizeof(szPath));
		GoToNewLocation(hDlg, szPath);
	}
}

static void ExploreDlg_OnSize(HWND hDlg, WORD wWidth, WORD wHeight)
{
	struct ExploreDlgData *lpData = (struct ExploreDlgData*) GetWindowLongPtr(hDlg, GWLP_USERDATA);

	RECT rect;
	GetWindowRect(GetDlgItem(hDlg, IDC_LOCATION), &rect);
	const int iLocationHeight = rect.bottom - rect.top;
	SetWindowPos(GetDlgItem(hDlg, IDC_LOCATION), NULL, 0, 0, wWidth, iLocationHeight, SWP_NOZORDER | SWP_NOMOVE);

	if (lpData->hListCtrl)
	{
		SetWindowPos(lpData->hListCtrl, NULL, 0, 0, wWidth, wHeight - iLocationHeight, SWP_NOZORDER | SWP_NOMOVE);
	}
}

static void ExploreDlg_OnNotify(HWND hDlg, LPARAM lParam)
{
	struct ExploreDlgData *lpData = (struct ExploreDlgData*) GetWindowLongPtr(hDlg, GWLP_USERDATA);
	LPNMHDR lpNmHdr = (LPNMHDR) lParam;

	if (lpNmHdr->hwndFrom == lpData->hListCtrl)
	{
		ExploreDlg_OnListCtrl(hDlg, lpNmHdr);
	}
}

static void ExploreDlg_OnDropFiles(HWND hDlg, HDROP hDrop)
{
	UINT nDropCount = DragQueryFile(hDrop, 0xFFFFFFFF, NULL, 0);

	for (UINT nIndex = 0; nIndex < nDropCount; nIndex++)
	{
		char szFile[MAX_PATH];

		if (DragQueryFile(hDrop, nIndex, szFile, sizeof(szFile)))
		{
			QuerySendFile(hDlg, szFile);
		}
	}

	DragFinish(hDrop);
}

static void ExploreDlg_OnClose(HWND hDlg)
{
	EndDialog(hDlg, IDCANCEL);
}

static void ExploreDlg_OnDestroy(HWND hDlg)
{
	struct ExploreDlgData *lpData = (struct ExploreDlgData*) GetWindowLongPtr(hDlg, GWLP_USERDATA);

	if (lpData->hListCtrl != NULL)
	{
		CloseWindow(lpData->hListCtrl);
		lpData->hListCtrl = NULL;
	}
}

static const CTRL_HANDLER ExploreDlgProc_Handler[] = {
	{ IDOK, ExploreDlg_OnOK },
};

INT_PTR CALLBACK ExploreDlgProc(HWND hDlg, UINT uMsg, WPARAM wParam, LPARAM lParam)
{
	BOOL fResult = FALSE;

	switch (uMsg)
	{
	case WM_INITDIALOG:
		// [...] return TRUE to direct the system to set the keyboard focus [...]
		ExploreDlg_OnInitDialog(hDlg, wParam, lParam);
		fResult = TRUE;
		break;
	case WM_COMMAND:
		// If an application processes this message, it should return zero.
		HandleControlMessage(hDlg, wParam, ExploreDlgProc_Handler, ARRAYSIZE(ExploreDlgProc_Handler));
		break;
	case WM_SIZE:
		ExploreDlg_OnSize(hDlg, LOWORD(lParam), HIWORD(lParam));
		break;
	case WM_NOTIFY:
		ExploreDlg_OnNotify(hDlg, lParam);
		break;
	case WM_DROPFILES:
		ExploreDlg_OnDropFiles(hDlg, (HDROP) wParam);
		break;
	case WM_CLOSE:
		// If an application processes this message, it should return zero.
		ExploreDlg_OnClose(hDlg);
		break;
	case WM_DESTROY:
		ExploreDlg_OnDestroy(hDlg);
		break;
	}

	return fResult;
}
