#include "fatutils.h"

#define BPB_OFFSET_NUMBER_OF_SECTORS  0x013  // 2 bytes
#define BPB_OFFSET_MEDIA_DESCRIPTOR   0x015  // 1 byte
#define BPB_OFFSET_SECTORS_PER_TRACK  0x018  // 2 bytes
#define BPB_OFFSET_NUMBER_OF_HEADS    0x01A  // 2 bytes

static const BYTE DriveTypeMediaDescriptors[] = {
	0x00,  // undefined
	0xfd,  // 360K
	0xf9,  // 1.2M
	0xf9,  // 720K
	0xf0,  // 1.44M
};

struct FatInfo GetFatInfoFromDrive(const struct DriveInfo *lpDriveInfo)
{
	struct FatInfo result;

	result.wNumberOfSectors = GetTotalSectorCount(lpDriveInfo);
	result.bMediaDescriptor = GetMediaDescriptorFromDriveType(lpDriveInfo->bDriveType);
	result.wSectorsPerTrack = lpDriveInfo->bSectorsPerTrack;
	result.wNumberOfHeads = lpDriveInfo->bNumberOfSides;

	return result;
}

struct FatInfo GetFatInfoFromImage(const BYTE SectorData[SECTOR_SIZE])
{
	struct FatInfo result;

	result.wNumberOfSectors = *(WORD*) &SectorData[BPB_OFFSET_NUMBER_OF_SECTORS];
	result.bMediaDescriptor = *(BYTE*) &SectorData[BPB_OFFSET_MEDIA_DESCRIPTOR];
	result.wSectorsPerTrack = *(WORD*) &SectorData[BPB_OFFSET_SECTORS_PER_TRACK];
	result.wNumberOfHeads   = *(WORD*) &SectorData[BPB_OFFSET_NUMBER_OF_HEADS];

	return result;
}

BYTE GetMediaDescriptorFromDriveType(BYTE bDriveType)
{
	return DriveTypeMediaDescriptors[(bDriveType < ARRAYSIZE(DriveTypeMediaDescriptors)) ? bDriveType : 0];
}

struct FatInfo GetResizedFatInfo(const struct FatInfo *lpSourceInfo, const struct FatInfo *lpTargetInfo)
{
	struct FatInfo result;

	result.wNumberOfSectors = min(lpSourceInfo->wNumberOfSectors, lpTargetInfo->wNumberOfSectors);
	result.bMediaDescriptor = lpTargetInfo->bMediaDescriptor;
	result.wSectorsPerTrack = lpTargetInfo->wSectorsPerTrack;
	result.wNumberOfHeads = lpTargetInfo->wNumberOfHeads;

	return result;
}

void SetFatInfoInImage(BYTE SectorData[SECTOR_SIZE], const struct FatInfo *lpFatInfo)
{
	*(WORD*) &SectorData[BPB_OFFSET_NUMBER_OF_SECTORS] = lpFatInfo->wNumberOfSectors;
	*(BYTE*) &SectorData[BPB_OFFSET_MEDIA_DESCRIPTOR]  = lpFatInfo->bMediaDescriptor;
	*(WORD*) &SectorData[BPB_OFFSET_SECTORS_PER_TRACK] = lpFatInfo->wSectorsPerTrack;
	*(WORD*) &SectorData[BPB_OFFSET_NUMBER_OF_HEADS]   = lpFatInfo->wNumberOfHeads;
}
