#include <windows.h>

#include "bgjob.h"
#include "resource.h"
#include "guiutils.h"

#include "findprogdlg.h"

static void FindProgDlg_OnInitDialog(HWND hDlg, WPARAM wParam, LPARAM lParam)
{
	SetWindowLongPtr(hDlg, GWLP_USERDATA, lParam);
	struct FindProgDlgData *lpData = (struct FindProgDlgData*) lParam;

	lpData->lpUpdateProgressProc(hDlg, lpData);
	SetBackgroundJobOwner(lpData->lpJob, hDlg);
	StartBackgroundJob(lpData->lpJob);
}

static void FindProgDlg_OnJobProgress(HWND hDlg)
{
	struct FindProgDlgData *lpData = (struct FindProgDlgData*) GetWindowLongPtr(hDlg, GWLP_USERDATA);
	lpData->lpUpdateProgressProc(hDlg, lpData);
}

static void FindProgDlg_OnCancel(HWND hDlg, WPARAM wParam)
{
	SendMessage(hDlg, WM_CLOSE, 0, 0);
}

static void FindProgDlg_OnClose(HWND hDlg)
{
	struct FindProgDlgData *lpData = (struct FindProgDlgData*) GetWindowLongPtr(hDlg, GWLP_USERDATA);
	CancelBackgroundJob(lpData->lpJob);

	// Note: The system expects that WM_CLOSE is handled by a call to EndDialog.
	//       Failing to do so causes the system to post WM_CLOSE messages again and again.
	//       On the other hand the dialog result should be the job result, so it cannot be
	//       ended before the job result code is known.
	//       By waiting here, the main thread is blocked until the job thread has posted
	//       the WM_JOB_FINISHED message, which triggers a call to EndDialog as well,
	//       with the result code to be passed to the dialog owner.
	WaitForBackgroundJob(lpData->lpJob, INFINITE);
}

static void FindProgDlg_OnJobFinished(HWND hDlg, WORD wResult)
{
	EndDialog(hDlg, wResult);
}

static const CTRL_HANDLER FindProgDlgProc_Handler[] = {
	{ IDCANCEL, FindProgDlg_OnCancel },
};

INT_PTR CALLBACK FindProgDlgProc(HWND hDlg, UINT uMsg, WPARAM wParam, LPARAM lParam)
{
	BOOL fResult = FALSE;

	switch (uMsg)
	{
	case WM_INITDIALOG:
		// [...] return TRUE to direct the system to set the keyboard focus [...]
		FindProgDlg_OnInitDialog(hDlg, wParam, lParam);
		fResult = TRUE;
		break;
	case WM_COMMAND:
		// If an application processes this message, it should return zero.
		HandleControlMessage(hDlg, wParam, FindProgDlgProc_Handler, ARRAYSIZE(FindProgDlgProc_Handler));
		break;
	case WM_JOB_PROGRESS:
		FindProgDlg_OnJobProgress(hDlg);
		break;
	case WM_JOB_FINISHED:
		FindProgDlg_OnJobFinished(hDlg, (WORD) wParam);
		break;
	case WM_CLOSE:
		// If an application processes this message, it should return zero.
		FindProgDlg_OnClose(hDlg);
		break;
	}

	return fResult;
}
