#include <stdio.h>
#include <stdlib.h>
#include <windows.h>

#include "settings.h"
#include "resource.h"
#include "guiutils.h"

#include "debug.h"

#include "flowctrldlg.h"

static void UpdateControls(HWND hDlg, const struct Settings *lpSettings)
{
	CheckDlgButton(hDlg, IDC_OUTX_CTS_FLOW, lpSettings->fOutxCtsFlow ? BST_CHECKED : BST_UNCHECKED);
	CheckDlgButton(hDlg, IDC_OUTX_DSR_FLOW, lpSettings->fOutxDsrFlow ? BST_CHECKED : BST_UNCHECKED);

	const struct OptionList *list;
	list = GetDtrControlSettings();
	PopulateComboBoxDword(GetDlgItem(hDlg, IDC_DTR_CONTROL), list, lpSettings->bDtrControl);
	list = GetRtsControlSettings();
	PopulateComboBoxDword(GetDlgItem(hDlg, IDC_RTS_CONTROL), list, lpSettings->bRtsControl);
}

static void FlowCtrlDlg_OnInitDialog(HWND hDlg, WPARAM wParam, LPARAM lParam)
{
	SetWindowLongPtr(hDlg, GWLP_USERDATA, lParam);
	const struct Settings *lpSettings = (struct Settings*) lParam;

	CenterWindow(hDlg, GetParent(hDlg));
	UpdateControls(hDlg, lpSettings);
}

static void FlowCtrlDlg_OnOK(HWND hDlg, WPARAM wParam)
{
	struct Settings *lpSettings = (struct Settings*) GetWindowLongPtr(hDlg, GWLP_USERDATA);

	DWORD dwValue;

	lpSettings->fOutxCtsFlow = IsDlgButtonChecked(hDlg, IDC_OUTX_CTS_FLOW) == BST_CHECKED;
	lpSettings->fOutxDsrFlow = IsDlgButtonChecked(hDlg, IDC_OUTX_DSR_FLOW) == BST_CHECKED;
	if (GetComboBoxSelectionDword(GetDlgItem(hDlg, IDC_DTR_CONTROL), &dwValue))
		lpSettings->bDtrControl = (BYTE) dwValue;
	if (GetComboBoxSelectionDword(GetDlgItem(hDlg, IDC_RTS_CONTROL), &dwValue))
		lpSettings->bRtsControl = (BYTE) dwValue;

	EndDialog(hDlg, IDOK);
}

static void FlowCtrlDlg_OnLoadDefaults(HWND hDlg, WPARAM wParam)
{
	struct Settings defaults;
	LoadDefaultSettings(&defaults, SETTINGS_FLOWCTRL);
	UpdateControls(hDlg, &defaults);
}

static void FlowCtrlDlg_OnCancel(HWND hDlg, WPARAM wParam)
{
	SendMessage(hDlg, WM_CLOSE, 0, 0);
}

static void FlowCtrlDlg_OnClose(HWND hDlg)
{
	EndDialog(hDlg, IDCANCEL);
}

static const CTRL_HANDLER FlowCtrlDlgProc_Handler[] = {
	{ IDOK,               FlowCtrlDlg_OnOK     },
	{ IDC_LOAD_DEFAULTS,  FlowCtrlDlg_OnLoadDefaults },
	{ IDCANCEL,           FlowCtrlDlg_OnCancel },
};

INT_PTR CALLBACK FlowCtrlDlgProc(HWND hDlg, UINT uMsg, WPARAM wParam, LPARAM lParam)
{
	BOOL fResult = FALSE;

	switch (uMsg)
	{
	case WM_INITDIALOG:
		// [...] return TRUE to direct the system to set the keyboard focus [...]
		FlowCtrlDlg_OnInitDialog(hDlg, wParam, lParam);
		fResult = TRUE;
		break;
	case WM_COMMAND:
		// If an application processes this message, it should return zero.
		HandleControlMessage(hDlg, wParam, FlowCtrlDlgProc_Handler, ARRAYSIZE(FlowCtrlDlgProc_Handler));
		break;
	case WM_CLOSE:
		// If an application processes this message, it should return zero.
		FlowCtrlDlg_OnClose(hDlg);
		break;
	}

	return fResult;
}
