#include <windows.h>
#include <stdlib.h>

#include "debug.h"
#include "list.h"

#define LIST_DEF_GROWTH  10

struct List {
	int iCount;
	int iCapacity;
	void **lpItems;
};

struct List *CreateList(void)
{
	struct List *lpList = MALLOC(sizeof(*lpList));

	if (lpList)
	{
		lpList->iCount = 0;
		lpList->iCapacity = 0;
		lpList->lpItems = NULL;
	}

	return lpList;
}

void DestroyList(struct List *lpList)
{
	ASSERT(lpList != NULL);

	if (lpList->lpItems)
	{
		for (int iIndex = 0; iIndex < lpList->iCount; iIndex++)
		{
			ASSERT(lpList->lpItems[iIndex] != NULL);
			FREE(lpList->lpItems[iIndex]);
		}
		FREE(lpList->lpItems);
	}
	FREE(lpList);
}

int GetListItemCount(const struct List *lpList)
{
	ASSERT(lpList != NULL);

	return lpList->iCount;
}

const void *GetListItem(const struct List *lpList, int iIndex)
{
	const void *lpResult = NULL;
	ASSERT(lpList != NULL);

	if (iIndex < lpList->iCount)
	{
		lpResult = lpList->lpItems[iIndex];
	}

	return lpResult;
}

int AddListItem(struct List *lpList, int iPosition, void *lpItem, int iItemSize)
{
	BOOL fFailed = FALSE;
	ASSERT(lpList != NULL);
	ASSERT(lpItem != NULL);
	ASSERT(iItemSize > 0);

	if (iPosition < 0 || iPosition > lpList->iCount)
		iPosition = lpList->iCount;

	if (lpList->iCount + 1 > lpList->iCapacity)
	{
		int iNewCapacity = lpList->iCapacity + LIST_DEF_GROWTH;
		void **lpNewItems = REALLOC(lpList->lpItems, iNewCapacity * sizeof(void*));

		if (lpNewItems)
		{
			lpList->iCapacity = iNewCapacity;
			lpList->lpItems = lpNewItems;
		}
		else
		{
			TRACE("Could not resize item list\n");
			fFailed = TRUE;
		}
	}

	void *lpNewItem = NULL;
	if (!fFailed)
	{
		lpNewItem = MALLOC(iItemSize);

		if (lpNewItem)
		{
			memcpy(lpNewItem, lpItem, iItemSize);
		}
		else
		{
			TRACE("Could not allocate memory for new list item\n");
			fFailed = TRUE;
		}
	}

	if (!fFailed)
	{
		for (int iNextFreeSlot = lpList->iCount; iNextFreeSlot > iPosition; iNextFreeSlot--)
		{
			lpList->lpItems[iNextFreeSlot] = lpList->lpItems[iNextFreeSlot - 1];
		}
		lpList->lpItems[iPosition] = lpNewItem;
		lpList->iCount++;
	}

	return fFailed ? -1 : iPosition;
}

void DelListItem(struct List *lpList, int iPosition)
{
	ASSERT(lpList != NULL);
	ASSERT(iPosition >= 0);
	ASSERT(iPosition < lpList->iCount);
	ASSERT(lpList->lpItems[iPosition] != NULL);

	if (iPosition >= 0)
	{
		for (int iEmptySlot = iPosition; iEmptySlot < lpList->iCount - 1; iEmptySlot++)
		{
			lpList->lpItems[iEmptySlot] = lpList->lpItems[iEmptySlot + 1];
		}
		lpList->iCount--;
	}
}

void SortList(struct List *lpList, int (*lpSortFunc)(const void *lpLeft, const void *lpRight))
{
	ASSERT(lpList != NULL);
	ASSERT(lpSortFunc != NULL);

	qsort(lpList->lpItems, lpList->iCount, sizeof(void*), lpSortFunc);
}
