#include <stdio.h>
#include <windows.h>
#include <commdlg.h>

#include "resource.h"
#include "guiutils.h"
#include "drivelist.h"

#include "connectdlg.h"
#include "delaysdlg.h"
#include "flowctrldlg.h"
#include "pickfiledlg.h"
#include "gettoolboxversion.h"
#include "exittoolbox.h"
#include "getdriveinfo.h"
#include "downloadtoolbox.h"
#include "transferimage.h"
#include "transferfile.h"
#include "driveeditdlg.h"
#include "exploredlg.h"

#include "debug.h"

#include "maindlg.h"

static void ClearDriveList(HWND hDriveList);
static void UpdateDriveInfo(HWND hDlg);

static void UpdateControlState(HWND hDlg)
{
	struct MainDlgData *lpData = (struct MainDlgData*) GetWindowLongPtr(hDlg, GWLP_USERDATA);

	BOOL fConnected = lpData->lpDevice != NULL;
	EnableMenuItem(GetMenu(hDlg), ID_FILE_CONNECT, !fConnected ? MF_ENABLED : MF_GRAYED);
	EnableMenuItem(GetMenu(hDlg), ID_FILE_DISCONNECT, fConnected ? MF_ENABLED : MF_GRAYED);
	EnableMenuItem(GetMenu(hDlg), ID_TOOLBOX_DOWNLOAD, fConnected ? MF_ENABLED : MF_GRAYED);
	EnableMenuItem(GetMenu(hDlg), ID_TOOLBOX_VERSION, fConnected ? MF_ENABLED : MF_GRAYED);
	EnableMenuItem(GetMenu(hDlg), ID_TOOLBOX_EXITTOLOADER, fConnected ? MF_ENABLED : MF_GRAYED);
	EnableMenuItem(GetMenu(hDlg), ID_TOOLBOX_EXITTOSYSTEM, fConnected ? MF_ENABLED : MF_GRAYED);
	EnableMenuItem(GetMenu(hDlg), ID_TWEAKS_DRIVEEDITOR, fConnected ? MF_ENABLED : MF_GRAYED);
	EnableMenuItem(GetMenu(hDlg), ID_TWEAKS_DELAYS, !fConnected ? MF_ENABLED : MF_GRAYED);
	EnableMenuItem(GetMenu(hDlg), ID_TWEAKS_FLOWCTRL, !fConnected ? MF_ENABLED : MF_GRAYED);

	int iCount = (int) SendMessage(GetDlgItem(hDlg, IDC_DRIVELIST), CB_GETCOUNT, 0, 0);
	EnableWindow(GetDlgItem(hDlg, IDC_DRIVELIST), fConnected && (iCount > 0));
	EnableWindow(GetDlgItem(hDlg, IDC_DETECT), fConnected);
	EnableWindow(GetDlgItem(hDlg, IDC_READIMAGE), fConnected && (iCount > 0));
	EnableWindow(GetDlgItem(hDlg, IDC_WRITEIMAGE), fConnected && (iCount > 0));
	EnableWindow(GetDlgItem(hDlg, IDC_EXPLORE), fConnected);
	EnableWindow(GetDlgItem(hDlg, IDC_RECVFILE), fConnected);
	EnableWindow(GetDlgItem(hDlg, IDC_SENDFILE), fConnected);
}

static void MainDlg_OnInitDialog(HWND hDlg, WPARAM wParam, LPARAM lParam)
{
	SetWindowLongPtr(hDlg, GWLP_USERDATA, lParam);
	CenterWindow(hDlg, GetParent(hDlg));
	UpdateControlState(hDlg);
}

static void MainDlg_OnFileConnect(HWND hDlg, WPARAM wParam)
{
	struct MainDlgData *lpData = (struct MainDlgData*) GetWindowLongPtr(hDlg, GWLP_USERDATA);
	HINSTANCE hInst = (HINSTANCE) GetWindowLongPtr(hDlg, GWLP_HINSTANCE);

	if (!lpData->lpDevice)
	{
		LoadSettings(&lpData->settings, SETTINGS_ALL);
		LPCSTR lpTemplate = MAKEINTRESOURCE(IDD_CONNECTDLG);
		INT_PTR iResult = DialogBoxParam(hInst, lpTemplate, hDlg, ConnectDlgProc, (LPARAM) &lpData->settings);
		if (iResult == IDOK)
		{
			SaveSettings(&lpData->settings, SETTINGS_CONNECT);
			lpData->lpDevice = OpenCommDev(&lpData->settings);
			if (!lpData->lpDevice)
			{
				FormatMessageBox(hDlg, MB_ICONERROR, "Error opening device %s: %s",
					lpData->settings.szPortName, GetLastErrorText());
			}
			UpdateControlState(hDlg);
		}
	}
}

static void MainDlg_OnFileDisconnect(HWND hDlg, WPARAM wParam)
{
	struct MainDlgData *lpData = (struct MainDlgData*) GetWindowLongPtr(hDlg, GWLP_USERDATA);

	if (lpData->lpDevice)
	{
		TRACE("Disconnect\n");
		CloseCommDev(lpData->lpDevice);
		lpData->lpDevice = NULL;
		ClearDriveList(GetDlgItem(hDlg, IDC_DRIVELIST));
		UpdateDriveInfo(hDlg);
		UpdateControlState(hDlg);
	}
}

static void MainDlg_OnFileExit(HWND hDlg, WPARAM wParam)
{
	SendMessage(hDlg, WM_CLOSE, 0, 0);
}

static void MainDlg_OnClose(HWND hDlg)
{
	ClearDriveList(GetDlgItem(hDlg, IDC_DRIVELIST));
	EndDialog(hDlg, 0);
}

static void MainDlg_OnToolboxDownload(HWND hDlg, WPARAM wParam)
{
	struct MainDlgData *lpData = (struct MainDlgData*) GetWindowLongPtr(hDlg, GWLP_USERDATA);

	OPENFILENAME_NT4 ofn = { 0 };
	char szFile[MAX_PATH] = { 0 };

	ofn.lStructSize = sizeof(ofn);
	ofn.hwndOwner = hDlg;
	ofn.lpstrFile = szFile;
	ofn.nMaxFile = sizeof(szFile);
	ofn.lpstrFilter = "All (*.*)\0*.*\0Binary files (*.bin)\0*.bin\0";
	ofn.nFilterIndex = 2;  // Index is 1-based
	ofn.Flags = OFN_PATHMUSTEXIST | OFN_FILEMUSTEXIST;

	if (GetOpenFileName((LPOPENFILENAME) &ofn))
	{
		DownloadToolBox(hDlg, lpData->lpDevice, szFile);
	}
}

static void MainDlg_OnToolboxVersion(HWND hDlg, WPARAM wParam)
{
	struct MainDlgData *lpData = (struct MainDlgData*) GetWindowLongPtr(hDlg, GWLP_USERDATA);

	WORD wVersion;
	if (GetToolboxVersion(hDlg, lpData->lpDevice, &wVersion))
	{
		FormatMessageBox(hDlg, MB_ICONINFORMATION, "Toolbox version %i.%i on target.", HIBYTE(wVersion), LOBYTE(wVersion));
	}
	else
	{
		FormatMessageBox(hDlg, MB_ICONERROR, "Get toolbox version failed.");
	}
}

static void MainDlg_OnToolboxExitToLoader(HWND hDlg, WPARAM wParam)
{
	struct MainDlgData *lpData = (struct MainDlgData*) GetWindowLongPtr(hDlg, GWLP_USERDATA);
	ExitToolbox(hDlg, lpData->lpDevice, 0);
}

static void MainDlg_OnToolboxExitToSystem(HWND hDlg, WPARAM wParam)
{
	struct MainDlgData *lpData = (struct MainDlgData*) GetWindowLongPtr(hDlg, GWLP_USERDATA);
	ExitToolbox(hDlg, lpData->lpDevice, 1);
}

static BOOL LoadTemporaryDriveList(HWND hDriveList, struct DriveEditDlgData *lpDlgData)
{
	BOOL fFailed = FALSE;

	const int iEntryCount = (int) SendMessage(hDriveList, CB_GETCOUNT, 0, 0);
	for (int iEntryIndex = 0; iEntryIndex < ARRAYSIZE(lpDlgData->lpEntries); iEntryIndex++)
	{
		struct DriveListEntry *lpNewEntry = NULL;

		if (!fFailed && iEntryIndex < iEntryCount)
		{
			const struct DriveListEntry *lpOldEntry = (struct DriveListEntry*) SendMessage(hDriveList, CB_GETITEMDATA, iEntryIndex, 0);
			fFailed = lpOldEntry == NULL;

			if (!fFailed)
			{
				lpNewEntry = MALLOC(sizeof(*lpNewEntry));
				fFailed = lpNewEntry == NULL;
			}

			if (!fFailed)
			{
				*lpNewEntry = *lpOldEntry;
			}
		}

		lpDlgData->lpEntries[iEntryIndex] = lpNewEntry;
	}

	return !fFailed;
}

static void StoreTemporaryDriveList(struct DriveEditDlgData *lpDlgData, HWND hDriveList)
{
	for (int iDriveIndex = 0; iDriveIndex < ARRAYSIZE(lpDlgData->lpEntries); iDriveIndex++)
	{
		if (lpDlgData->lpEntries[iDriveIndex])
		{
			char szName[MAX_DRIVE_NAME];
			sprintf(szName, "Drive %i", lpDlgData->lpEntries[iDriveIndex]->bDriveIndex);
			int iItemIndex = (int) SendMessage(hDriveList, CB_ADDSTRING, 0, (LPARAM) szName);
			ASSERT(iItemIndex != CB_ERR);
			SendMessage(hDriveList, CB_SETITEMDATA, iItemIndex, (LPARAM) lpDlgData->lpEntries[iDriveIndex]);
			lpDlgData->lpEntries[iDriveIndex] = NULL;
		}
	}
}

static void FreeTemporaryDriveList(struct DriveEditDlgData *lpDlgData)
{
	for (int iEntryIndex = 0; iEntryIndex < ARRAYSIZE(lpDlgData->lpEntries); iEntryIndex++)
	{
		if (lpDlgData->lpEntries[iEntryIndex])
		{
			FREE(lpDlgData->lpEntries[iEntryIndex]);
			lpDlgData->lpEntries[iEntryIndex] = NULL;
		}
	}
}

static void MainDlg_OnTweaksDriveEditor(HWND hDlg, WPARAM wParam)
{
	BOOL fFailed = FALSE;
	struct DriveEditDlgData DlgParam;

	if (LoadTemporaryDriveList(GetDlgItem(hDlg, IDC_DRIVELIST), &DlgParam) == FALSE)
	{
		FormatMessageBox(hDlg, MB_ICONERROR, "Error creating temporary drive list, out of memory?");
		fFailed = TRUE;
	}

	if (!fFailed)
	{
		HINSTANCE hInst = (HINSTANCE) GetWindowLongPtr(hDlg, GWLP_HINSTANCE);
		LPCSTR lpTemplate = MAKEINTRESOURCE(IDD_DRIVEEDITDLG);
		INT_PTR iResult = DialogBoxParam(hInst, lpTemplate, hDlg, DriveEditDlgProc, (LPARAM) &DlgParam);
		if (iResult == IDOK)
		{
			ClearDriveList(GetDlgItem(hDlg, IDC_DRIVELIST));
			StoreTemporaryDriveList(&DlgParam, GetDlgItem(hDlg, IDC_DRIVELIST));
			SendMessage(GetDlgItem(hDlg, IDC_DRIVELIST), CB_SETCURSEL, 0, 0);
			UpdateDriveInfo(hDlg);
			UpdateControlState(hDlg);
		}
	}

	FreeTemporaryDriveList(&DlgParam);
}

static void MainDlg_OnTweaksDelays(HWND hDlg, WPARAM wParam)
{
	struct MainDlgData *lpData = (struct MainDlgData*) GetWindowLongPtr(hDlg, GWLP_USERDATA);
	HINSTANCE hInst = (HINSTANCE) GetWindowLongPtr(hDlg, GWLP_HINSTANCE);

	if (!lpData->lpDevice)
	{
		LoadSettings(&lpData->settings, SETTINGS_DELAYS);
		LPCSTR lpTemplate = MAKEINTRESOURCE(IDD_DELAYSDLG);
		INT_PTR iResult = DialogBoxParam(hInst, lpTemplate, hDlg, DelaysDlgProc, (LPARAM) &lpData->settings);
		if (iResult == IDOK)
		{
			SaveSettings(&lpData->settings, SETTINGS_DELAYS);
		}
	}
}

static void MainDlg_OnTweaksFlowCtrl(HWND hDlg, WPARAM wParam)
{
	struct MainDlgData *lpData = (struct MainDlgData*) GetWindowLongPtr(hDlg, GWLP_USERDATA);
	HINSTANCE hInst = (HINSTANCE) GetWindowLongPtr(hDlg, GWLP_HINSTANCE);

	if (!lpData->lpDevice)
	{
		LoadSettings(&lpData->settings, SETTINGS_FLOWCTRL);
		LPCSTR lpTemplate = MAKEINTRESOURCE(IDD_FLOWCTRLDLG);
		INT_PTR iResult = DialogBoxParam(hInst, lpTemplate, hDlg, FlowCtrlDlgProc, (LPARAM) &lpData->settings);
		if (iResult == IDOK)
		{
			SaveSettings(&lpData->settings, SETTINGS_FLOWCTRL);
		}
	}
}

static void MainDlg_OnHelpAbout(HWND hDlg, WPARAM wParam)
{
	FormatMessageBox(hDlg, MB_ICONINFORMATION, "toolbox-client version %i.%i.\r\n\r\n"
		"Written by Felix Opatz <felix@zotteljedi.de>", VERSION_MAJOR, VERSION_MINOR);
}

static void ClearDriveList(HWND hDriveList)
{
	int iEntryCount = (int) SendMessage(hDriveList, CB_GETCOUNT, 0, 0);
	for (int iIndex = 0; iIndex < iEntryCount; iIndex++)
	{
		struct DriveListEntry *lpEntry = (struct DriveListEntry*) SendMessage(hDriveList, CB_GETITEMDATA, iIndex, 0);
		ASSERT(lpEntry != NULL);
		if (lpEntry)
			FREE(lpEntry);
	}
	SendMessage(hDriveList, CB_RESETCONTENT, 0, 0);
}

// Code duplication with driveeditdlg.c
static void UpdateDriveInfo(HWND hDlg)
{
	HWND hItem = GetDlgItem(hDlg, IDC_DRIVELIST);
	int iSel = (int) SendMessage(hItem, CB_GETCURSEL, 0, 0);
	if (iSel != CB_ERR)
	{
		struct DriveListEntry *lpEntry = (struct DriveListEntry*) SendMessage(hItem, CB_GETITEMDATA, iSel, 0);
		ASSERT(lpEntry != NULL);
		if (lpEntry)
		{
			SetWindowText(GetDlgItem(hDlg, IDC_TYPE), GetDriveTypeString(lpEntry->Info.bDriveType));
			char szText[MAX_LABEL];
			sprintf(szText, "%i", lpEntry->Info.bNumberOfTracks);
			SetWindowText(GetDlgItem(hDlg, IDC_TRACKS), szText);
			sprintf(szText, "%i", lpEntry->Info.bNumberOfSides);
			SetWindowText(GetDlgItem(hDlg, IDC_SIDES), szText);
			sprintf(szText, "%i", lpEntry->Info.bSectorsPerTrack);
			SetWindowText(GetDlgItem(hDlg, IDC_SECTORS), szText);
		}
	}
	else
	{
		SetWindowText(GetDlgItem(hDlg, IDC_TYPE), "-");
		SetWindowText(GetDlgItem(hDlg, IDC_TRACKS), "-");
		SetWindowText(GetDlgItem(hDlg, IDC_SIDES), "-");
		SetWindowText(GetDlgItem(hDlg, IDC_SECTORS), "-");
	}
}

static void MainDlg_OnDetect(HWND hDlg, WPARAM wParam)
{
	struct MainDlgData *lpData = (struct MainDlgData*) GetWindowLongPtr(hDlg, GWLP_USERDATA);

	ClearDriveList(GetDlgItem(hDlg, IDC_DRIVELIST));

	int iDrivesFound = 0;
	for (int iDriveIndex = 0; iDriveIndex < MAX_DRIVES; iDriveIndex++)
	{
		struct DriveInfo info;
		if (GetDriveInfo(hDlg, lpData->lpDevice, iDriveIndex, &info))
		{
			struct DriveListEntry *lpEntry = MALLOC(sizeof(*lpEntry));
			if (lpEntry)
			{
				lpEntry->bDriveIndex = iDriveIndex;
				lpEntry->Info = info;

				char szName[MAX_DRIVE_NAME];
				sprintf(szName, "Drive %i", iDriveIndex);
				int iItemIndex = (int) SendMessage(GetDlgItem(hDlg, IDC_DRIVELIST), CB_ADDSTRING, 0, (LPARAM) szName);
				ASSERT(iItemIndex != CB_ERR);
				SendMessage(GetDlgItem(hDlg, IDC_DRIVELIST), CB_SETITEMDATA, iItemIndex, (LPARAM) lpEntry);
				iDrivesFound++;
			}
			else
			{
				FormatMessageBox(hDlg, MB_ICONERROR, "Cannot store drive info, out of memory?");
				break;
			}
		}
		else
		{
			TRACE("GetDriveInfo failed for drive index %i, aborting loop.\n", iDriveIndex);
			break;
		}
	}

	if (iDrivesFound == 0)
	{
		FormatMessageBox(hDlg, MB_ICONINFORMATION, "Unable to detect drives automatically.\n"
			"Use the drive editor to add drives manually.");
	}

	SendMessage(GetDlgItem(hDlg, IDC_DRIVELIST), CB_SETCURSEL, 0, 0);
	UpdateControlState(hDlg);
	UpdateDriveInfo(hDlg);
}

static void MainDlg_OnDriveList(HWND hDlg, WPARAM wParam)
{
	if (HIWORD(wParam) == CBN_SELCHANGE)
		UpdateDriveInfo(hDlg);
}

static void MainDlg_OnReadImage(HWND hDlg, WPARAM wParam)
{
	struct MainDlgData *lpData = (struct MainDlgData*) GetWindowLongPtr(hDlg, GWLP_USERDATA);

	HWND hItem = GetDlgItem(hDlg, IDC_DRIVELIST);
	int iSel = (int) SendMessage(hItem, CB_GETCURSEL, 0, 0);
	if (iSel != CB_ERR)
	{
		struct DriveListEntry *lpEntry = (struct DriveListEntry*) SendMessage(hItem, CB_GETITEMDATA, iSel, 0);
		ASSERT(lpEntry != NULL);

		OPENFILENAME_NT4 ofn = { 0 };
		char szFile[MAX_PATH] = { 0 };

		ofn.lStructSize = sizeof(ofn);
		ofn.hwndOwner = hDlg;
		ofn.lpstrFile = szFile;
		ofn.nMaxFile = sizeof(szFile);
		ofn.lpstrFilter = "All (*.*)\0*.*\0Image files (*.img)\0*.img\0";
		ofn.nFilterIndex = 2;  // Index is 1-based
		ofn.Flags = OFN_OVERWRITEPROMPT;

		if (GetSaveFileName((LPOPENFILENAME) &ofn))
		{
			if (ReadImage(hDlg, lpData->lpDevice, lpEntry->bDriveIndex, &lpEntry->Info, szFile))
			{
				FormatMessageBox(hDlg, MB_ICONINFORMATION, "Read disk image succeeded.");
			}
			else
			{
				FormatMessageBox(hDlg, MB_ICONERROR, "Read disk image failed.");
			}
		}
	}
}

static void MainDlg_OnWriteImage(HWND hDlg, WPARAM wParam)
{
	struct MainDlgData *lpData = (struct MainDlgData*) GetWindowLongPtr(hDlg, GWLP_USERDATA);

	HWND hItem = GetDlgItem(hDlg, IDC_DRIVELIST);
	int iSel = (int) SendMessage(hItem, CB_GETCURSEL, 0, 0);
	if (iSel != CB_ERR)
	{
		struct DriveListEntry *lpEntry = (struct DriveListEntry*) SendMessage(hItem, CB_GETITEMDATA, iSel, 0);
		ASSERT(lpEntry != NULL);

		OPENFILENAME_NT4 ofn = { 0 };
		char szFile[MAX_PATH] = { 0 };

		ofn.lStructSize = sizeof(ofn);
		ofn.hwndOwner = hDlg;
		ofn.lpstrFile = szFile;
		ofn.nMaxFile = sizeof(szFile);
		ofn.lpstrFilter = "All (*.*)\0*.*\0Image files (*.img)\0*.img\0";
		ofn.nFilterIndex = 2;  // Index is 1-based
		ofn.Flags = OFN_PATHMUSTEXIST | OFN_FILEMUSTEXIST;

		if (GetOpenFileName((LPOPENFILENAME) &ofn))
		{
			if (WriteImage(hDlg, lpData->lpDevice, lpEntry->bDriveIndex, &lpEntry->Info, szFile))
			{
				FormatMessageBox(hDlg, MB_ICONINFORMATION, "Write disk image succeeded.");
			}
			else
			{
				FormatMessageBox(hDlg, MB_ICONERROR, "Write disk image failed.");
			}
		}
	}
}

static void MainDlg_OnExplore(HWND hDlg, WPARAM wParam)
{
	struct MainDlgData *lpData = (struct MainDlgData*) GetWindowLongPtr(hDlg, GWLP_USERDATA);
	HINSTANCE hInst = (HINSTANCE) GetWindowLongPtr(hDlg, GWLP_HINSTANCE);
	LPCSTR lpTemplate = MAKEINTRESOURCE(IDD_EXPLOREDLG);

	struct ExploreDlgData DlgParam = { 0 };
	DlgParam.lpDevice = lpData->lpDevice;
	DialogBoxParam(hInst, lpTemplate, hDlg, ExploreDlgProc, (LPARAM) &DlgParam);
}

static void MainDlg_OnRecvFile(HWND hDlg, WPARAM wParam)
{
	struct MainDlgData *lpData = (struct MainDlgData*) GetWindowLongPtr(hDlg, GWLP_USERDATA);
	HINSTANCE hInst = (HINSTANCE) GetWindowLongPtr(hDlg, GWLP_HINSTANCE);

	LPCSTR lpTemplate = MAKEINTRESOURCE(IDD_FILEPICKDLG);
	struct PickFileDlgParam DlgParam = { 0 };
	DlgParam.bMode = PFD_MODE_RECEIVE;
	INT_PTR iResult = DialogBoxParam(hInst, lpTemplate, hDlg, PickFileDlgProc, (LPARAM) &DlgParam);
	if (iResult == IDOK)
	{
		if (RecvFile(hDlg, lpData->lpDevice, DlgParam.szRemotePath, DlgParam.szLocalPath))
		{
			FormatMessageBox(hDlg, MB_ICONINFORMATION, "Receive file succeeded.");
		}
		else
		{
			FormatMessageBox(hDlg, MB_ICONERROR, "Receive file failed.");
		}
	}
}

static void MainDlg_OnSendFile(HWND hDlg, WPARAM wParam)
{
	struct MainDlgData *lpData = (struct MainDlgData*) GetWindowLongPtr(hDlg, GWLP_USERDATA);
	HINSTANCE hInst = (HINSTANCE) GetWindowLongPtr(hDlg, GWLP_HINSTANCE);

	LPCSTR lpTemplate = MAKEINTRESOURCE(IDD_FILEPICKDLG);
	struct PickFileDlgParam DlgParam = { 0 };
	DlgParam.bMode = PFD_MODE_SEND;
	INT_PTR iResult = DialogBoxParam(hInst, lpTemplate, hDlg, PickFileDlgProc, (LPARAM) &DlgParam);
	if (iResult == IDOK)
	{
		if (SendFile(hDlg, lpData->lpDevice, DlgParam.szLocalPath, DlgParam.szRemotePath))
		{
			FormatMessageBox(hDlg, MB_ICONINFORMATION, "Send file succeeded.");
		}
		else
		{
			FormatMessageBox(hDlg, MB_ICONERROR, "Send file failed.");
		}
	}
}

static const CTRL_HANDLER MainDlgProc_Handler[] = {
	{ ID_FILE_CONNECT,          MainDlg_OnFileConnect         },
	{ ID_FILE_DISCONNECT,       MainDlg_OnFileDisconnect      },
	{ ID_FILE_EXIT,             MainDlg_OnFileExit            },
	{ ID_TOOLBOX_DOWNLOAD,      MainDlg_OnToolboxDownload     },
	{ ID_TOOLBOX_VERSION,       MainDlg_OnToolboxVersion      },
	{ ID_TOOLBOX_EXITTOLOADER,  MainDlg_OnToolboxExitToLoader },
	{ ID_TOOLBOX_EXITTOSYSTEM,  MainDlg_OnToolboxExitToSystem },
	{ ID_TWEAKS_DRIVEEDITOR,    MainDlg_OnTweaksDriveEditor   },
	{ ID_TWEAKS_DELAYS,         MainDlg_OnTweaksDelays        },
	{ ID_TWEAKS_FLOWCTRL,       MainDlg_OnTweaksFlowCtrl      },
	{ ID_HELP_ABOUT,            MainDlg_OnHelpAbout           },
	{ IDC_DETECT,               MainDlg_OnDetect              },
	{ IDC_DRIVELIST,            MainDlg_OnDriveList           },
	{ IDC_READIMAGE,            MainDlg_OnReadImage           },
	{ IDC_WRITEIMAGE,           MainDlg_OnWriteImage          },
	{ IDC_EXPLORE,              MainDlg_OnExplore             },
	{ IDC_RECVFILE,             MainDlg_OnRecvFile            },
	{ IDC_SENDFILE,             MainDlg_OnSendFile            },
};

void InitMainDlgData(struct MainDlgData *lpData)
{
	lpData->lpDevice = NULL;
}

void CleanUpMainDlgData(struct MainDlgData *lpData)
{
	if (lpData->lpDevice)
	{
		CloseCommDev(lpData->lpDevice);
		lpData->lpDevice = NULL;
	}
}

INT_PTR CALLBACK MainDlgProc(HWND hDlg, UINT uMsg, WPARAM wParam, LPARAM lParam)
{
	BOOL fResult = FALSE;

	switch (uMsg)
	{
	case WM_INITDIALOG:
		// [...] return TRUE to direct the system to set the keyboard focus [...]
		MainDlg_OnInitDialog(hDlg, wParam, lParam);
		fResult = TRUE;
		break;
	case WM_COMMAND:
		// If an application processes this message, it should return zero.
		HandleControlMessage(hDlg, wParam, MainDlgProc_Handler, ARRAYSIZE(MainDlgProc_Handler));
		break;
	case WM_CLOSE:
		// If an application processes this message, it should return zero.
		MainDlg_OnClose(hDlg);
		break;
	}

	return fResult;
}
