#!/usr/bin/perl

use strict;
use Time::HiRes;

my $path360kImage = "../test-images/360k.img";
my $path720kImage = "../test-images/720k.img";
my $path1200kImage = "../test-images/1200k.img";
my $path1440kImage = "../test-images/1440k.img";
my $size360k = 368640;
my $size720k = 737280;
my $size1200k = 1228800;
my $size1440k = 1474560;

my $binPath = "src/fat12-resize/fat12-resize.exe";
my $runCmd = "wine";
my $workDir = "fat12-resize-test-work";
my $workDirCreated;

sub checkEnvironment
{
    if (not -e $workDir)
    {
        print "workdir not found at $workDir, creating it.\n";
        mkdir $workDir or die "error creating workDir";
        mkdir "$workDir/mnt1" or die "error creating mount point 1";
        mkdir "$workDir/mnt2" or die "error creating mount point 2";
        $workDirCreated = 1;
    }

    -e $path360kImage or die "360k image not found at $path360kImage";
    -s $path360kImage == $size360k or die "360k image at $path360kImage has wrong size";
    -e $path720kImage or die "720k image not found at $path720kImage";
    -s $path720kImage == $size720k or die "720k image at $path720kImage has wrong size";
    -e $path1200kImage or die "1200k image not found at $path1200kImage";
    -s $path1200kImage == $size1200k or die "1200k image at $path1200kImage has wrong size";
    -e $path1440kImage or die "1440k image not found at $path1440kImage";
    -s $path1440kImage == $size1440k or die "1440k image at $path1440kImage has wrong size";
}

sub cleanEnvironment
{
    if ($workDirCreated)
    {
        print "removing workdir\n";
        system "rm -rf $workDir";
    }
}

sub createVariants
{
    system "$runCmd $binPath $path360kImage $workDir/360k-as-360k.img 1";
    system "$runCmd $binPath $path360kImage $workDir/360k-as-720k.img 3";
    system "$runCmd $binPath $path360kImage $workDir/360k-as-1200k.img 2";
    system "$runCmd $binPath $path360kImage $workDir/360k-as-1440k.img 4";

    system "$runCmd $binPath $path720kImage $workDir/720k-as-360k.img 1";
    system "$runCmd $binPath $path720kImage $workDir/720k-as-720k.img 3";
    system "$runCmd $binPath $path720kImage $workDir/720k-as-1200k.img 2";
    system "$runCmd $binPath $path720kImage $workDir/720k-as-1440k.img 4";

    system "$runCmd $binPath $path1200kImage $workDir/1200k-as-360k.img 1";
    system "$runCmd $binPath $path1200kImage $workDir/1200k-as-720k.img 3";
    system "$runCmd $binPath $path1200kImage $workDir/1200k-as-1200k.img 2";
    system "$runCmd $binPath $path1200kImage $workDir/1200k-as-1440k.img 4";

    system "$runCmd $binPath $path1440kImage $workDir/1440k-as-360k.img 1";
    system "$runCmd $binPath $path1440kImage $workDir/1440k-as-720k.img 3";
    system "$runCmd $binPath $path1440kImage $workDir/1440k-as-1200k.img 2";
    system "$runCmd $binPath $path1440kImage $workDir/1440k-as-1440k.img 4";
}

sub checkImageSize
{
    my $file = $_[0];
    my $size = $_[1];
    my $failed = 0;

    if (-s "$file" != $size)
    {
        print "size of $file is wrong\n";
        $failed = 1;
    }

    return $failed;
}

sub checkSizes
{
    my $failed = 0;

    $failed += checkImageSize("$workDir/360k-as-360k.img", $size360k);
    $failed += checkImageSize("$workDir/720k-as-360k.img", $size360k);
    $failed += checkImageSize("$workDir/1200k-as-360k.img", $size360k);
    $failed += checkImageSize("$workDir/1440k-as-360k.img", $size360k);

    $failed += checkImageSize("$workDir/360k-as-720k.img", $size720k);
    $failed += checkImageSize("$workDir/720k-as-720k.img", $size720k);
    $failed += checkImageSize("$workDir/1200k-as-720k.img", $size720k);
    $failed += checkImageSize("$workDir/1440k-as-720k.img", $size720k);

    $failed += checkImageSize("$workDir/360k-as-1200k.img", $size1200k);
    $failed += checkImageSize("$workDir/720k-as-1200k.img", $size1200k);
    $failed += checkImageSize("$workDir/1200k-as-1200k.img", $size1200k);
    $failed += checkImageSize("$workDir/1440k-as-1200k.img", $size1200k);

    $failed += checkImageSize("$workDir/360k-as-1440k.img", $size1440k);
    $failed += checkImageSize("$workDir/720k-as-1440k.img", $size1440k);
    $failed += checkImageSize("$workDir/1200k-as-1440k.img", $size1440k);
    $failed += checkImageSize("$workDir/1440k-as-1440k.img", $size1440k);

    return $failed;
}

sub checkImageSame
{
    my $file1 = $_[0];
    my $file2 = $_[1];
    my $failed = 0;

    $failed = 1 if system("diff $file1 $file2") != 0;

    return $failed;
}

sub checkNeutral
{
    my $failed = 0;

    $failed += checkImageSame("$workDir/360k-as-360k.img", $path360kImage);
    $failed += checkImageSame("$workDir/720k-as-720k.img", $path720kImage);
    $failed += checkImageSame("$workDir/1200k-as-1200k.img", $path1200kImage);
    $failed += checkImageSame("$workDir/1440k-as-1440k.img", $path1440kImage);
}

sub createReverse
{
    system "$runCmd $binPath $workDir/360k-as-360k.img $workDir/360k-as-360k-as-360k.img 1";
    system "$runCmd $binPath $workDir/360k-as-720k.img $workDir/360k-as-720k-as-360k.img 1";
    system "$runCmd $binPath $workDir/360k-as-1200k.img $workDir/360k-as-1200k-as-360k.img 1";
    system "$runCmd $binPath $workDir/360k-as-1440k.img $workDir/360k-as-1440k-as-360k.img 1";

    system "$runCmd $binPath $workDir/720k-as-720k.img $workDir/720k-as-720k-as-720k.img 3";
    system "$runCmd $binPath $workDir/720k-as-1200k.img $workDir/720k-as-1200k-as-720k.img 3";
    system "$runCmd $binPath $workDir/720k-as-1440k.img $workDir/720k-as-1440k-as-720k.img 3";

    system "$runCmd $binPath $workDir/1200k-as-1200k.img $workDir/1200k-as-1200k-as-1200k.img 2";
    system "$runCmd $binPath $workDir/1200k-as-1440k.img $workDir/1200k-as-1440k-as-1200k.img 2";

    system "$runCmd $binPath $workDir/1440k-as-1440k.img $workDir/1440k-as-1440k-as-1440k.img 4";
}

sub checkReverse
{
    my $failed = 0;

    $failed += checkImageSame("$workDir/360k-as-360k-as-360k.img", $path360kImage);
    $failed += checkImageSame("$workDir/360k-as-720k-as-360k.img", $path360kImage);
    $failed += checkImageSame("$workDir/360k-as-1200k-as-360k.img", $path360kImage);
    $failed += checkImageSame("$workDir/360k-as-1440k-as-360k.img", $path360kImage);

    $failed += checkImageSame("$workDir/720k-as-720k-as-720k.img", $path720kImage);
    $failed += checkImageSame("$workDir/720k-as-1200k-as-720k.img", $path720kImage);
    $failed += checkImageSame("$workDir/720k-as-1440k-as-720k.img", $path720kImage);

    $failed += checkImageSame("$workDir/1200k-as-1200k-as-1200k.img", $path1200kImage);
    $failed += checkImageSame("$workDir/1200k-as-1440k-as-1200k.img", $path1200kImage);

    $failed += checkImageSame("$workDir/1440k-as-1440k-as-1440k.img", $path1440kImage);

    return $failed;
}

sub checkFileList
{
    my $file1 = $_[0];
    my $file2 = $_[1];
    my $failed = 0;

    system("sudo mount -o loop,ro $file1 $workDir/mnt1");
    system("sudo mount -o loop,ro $file2 $workDir/mnt2");

    my $list1 = `find "$workDir/mnt1" | sed s/mnt1//`;
    my $list2 = `find "$workDir/mnt2" | sed s/mnt2//`;

    if ($list1 ne $list2)
    {
        print "file lists for $file1 and $file2 differ\n";
        $failed = 1;
    }

    # If not sleeping here, umount sometimes fails because still in use
    Time::HiRes::usleep(50000);

    system("sudo umount $workDir/mnt1");
    system("sudo umount $workDir/mnt2");

    return $failed;
}

sub checkFileLists
{
    my $failed = 0;
    
    $failed += checkFileList("$workDir/360k-as-720k.img", $path360kImage);
    $failed += checkFileList("$workDir/360k-as-1200k.img", $path360kImage);
    $failed += checkFileList("$workDir/360k-as-1440k.img", $path360kImage);

    $failed += checkFileList("$workDir/720k-as-360k.img", $path720kImage);
    $failed += checkFileList("$workDir/720k-as-1200k.img", $path720kImage);
    $failed += checkFileList("$workDir/720k-as-1440k.img", $path720kImage);

    $failed += checkFileList("$workDir/1200k-as-360k.img", $path1200kImage);
    $failed += checkFileList("$workDir/1200k-as-720k.img", $path1200kImage);
    $failed += checkFileList("$workDir/1200k-as-1440k.img", $path1200kImage);

    $failed += checkFileList("$workDir/1440k-as-360k.img", $path1440kImage);
    $failed += checkFileList("$workDir/1440k-as-720k.img", $path1440kImage);
    $failed += checkFileList("$workDir/1440k-as-1200k.img", $path1440kImage);

    return $failed;
}

sub checkFileContent
{
    my $file1 = $_[0];
    my $file2 = $_[1];
    my $failed = 0;

    system("sudo mount -o loop,ro $file1 $workDir/mnt1");
    system("sudo mount -o loop,ro $file2 $workDir/mnt2");

    my @list1 = split(/\n/, `find "$workDir/mnt1"`);
    my @list2 = split(/\n/, `find "$workDir/mnt2"`);

    for (my $i = 0; $i < scalar @list1; $i++)
    {
        if (system("diff $list1[$i] $list2[$i]") != 0)
        {
            $failed = 1;
            last;
        }
    }

    # If not sleeping here, umount sometimes fails because still in use
    Time::HiRes::usleep(50000);

    system("sudo umount $workDir/mnt1");
    system("sudo umount $workDir/mnt2");

    return $failed;
}

sub checkFileContents
{
    my $failed = 0;
    
    $failed += checkFileContent("$workDir/360k-as-720k.img", $path360kImage);
    $failed += checkFileContent("$workDir/360k-as-1200k.img", $path360kImage);
    $failed += checkFileContent("$workDir/360k-as-1440k.img", $path360kImage);

    $failed += checkFileContent("$workDir/720k-as-1200k.img", $path720kImage);
    $failed += checkFileContent("$workDir/720k-as-1440k.img", $path720kImage);

    $failed += checkFileContent("$workDir/1200k-as-1440k.img", $path1200kImage);

    return $failed;
}

sub main
{
    my $failed = 0;

    checkEnvironment();
    createVariants();
    $failed += checkSizes();
    $failed += checkNeutral();
    createReverse();
    $failed += checkReverse();
    $failed += checkFileLists();
    $failed += checkFileContents();
    cleanEnvironment();

    print "$failed tests failed\n";
}

main();
